from enum import IntEnum
from typing import overload, NewType, Optional, Tuple

from OCC.Core.Standard import *
from OCC.Core.NCollection import *
from OCC.Core.TCollection import *
from OCC.Core.TColStd import *

#the following typedef cannot be wrapped as is
TDF_AttributeDoubleMap = NewType('TDF_AttributeDoubleMap', Any)
#the following typedef cannot be wrapped as is
TDF_AttributeIndexedMap = NewType('TDF_AttributeIndexedMap', Any)
#the following typedef cannot be wrapped as is
TDF_AttributeMap = NewType('TDF_AttributeMap', Any)
#the following typedef cannot be wrapped as is
TDF_DoubleMapIteratorOfAttributeDoubleMap = NewType('TDF_DoubleMapIteratorOfAttributeDoubleMap', Any)
#the following typedef cannot be wrapped as is
TDF_DoubleMapIteratorOfGUIDProgIDMap = NewType('TDF_DoubleMapIteratorOfGUIDProgIDMap', Any)
#the following typedef cannot be wrapped as is
TDF_DoubleMapIteratorOfLabelDoubleMap = NewType('TDF_DoubleMapIteratorOfLabelDoubleMap', Any)
#the following typedef cannot be wrapped as is
TDF_GUIDProgIDMap = NewType('TDF_GUIDProgIDMap', Any)
#the following typedef cannot be wrapped as is
TDF_HAllocator = NewType('TDF_HAllocator', Any)
#the following typedef cannot be wrapped as is
TDF_IDMap = NewType('TDF_IDMap', Any)
#the following typedef cannot be wrapped as is
TDF_LabelDoubleMap = NewType('TDF_LabelDoubleMap', Any)
#the following typedef cannot be wrapped as is
TDF_LabelIndexedMap = NewType('TDF_LabelIndexedMap', Any)
#the following typedef cannot be wrapped as is
TDF_LabelMap = NewType('TDF_LabelMap', Any)
TDF_LabelNodePtr = NewType('TDF_LabelNodePtr', TDF_LabelNode)
#the following typedef cannot be wrapped as is
TDF_MapIteratorOfAttributeMap = NewType('TDF_MapIteratorOfAttributeMap', Any)
#the following typedef cannot be wrapped as is
TDF_MapIteratorOfIDMap = NewType('TDF_MapIteratorOfIDMap', Any)
#the following typedef cannot be wrapped as is
TDF_MapIteratorOfLabelMap = NewType('TDF_MapIteratorOfLabelMap', Any)

class TDF_AttributeArray1:
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, theLower: int, theUpper: int) -> None: ...
    def __getitem__(self, index: int) -> False: ...
    def __setitem__(self, index: int, value: False) -> None: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[False]: ...
    def next(self) -> False: ...
    __next__ = next
    def Init(self, theValue: False) -> None: ...
    def Size(self) -> int: ...
    def Length(self) -> int: ...
    def IsEmpty(self) -> bool: ...
    def Lower(self) -> int: ...
    def Upper(self) -> int: ...
    def IsDetectable(self) -> bool: ...
    def IsAllocated(self) -> bool: ...
    def First(self) -> False: ...
    def Last(self) -> False: ...
    def Value(self, theIndex: int) -> False: ...
    def SetValue(self, theIndex: int, theValue: False) -> None: ...

class TDF_AttributeDeltaList:
    def __init__(self) -> None: ...
    def __len__(self) -> int: ...
    def Size(self) -> int: ...
    def Clear(self) -> None: ...
    def First(self) -> False: ...
    def Last(self) -> False: ...
    def Append(self, theItem: False) -> False: ...
    def Prepend(self, theItem: False) -> False: ...
    def RemoveFirst(self) -> None: ...
    def Reverse(self) -> None: ...
    def Value(self, theIndex: int) -> False: ...
    def SetValue(self, theIndex: int, theValue: False) -> None: ...

class TDF_AttributeList:
    def __init__(self) -> None: ...
    def __len__(self) -> int: ...
    def Size(self) -> int: ...
    def Clear(self) -> None: ...
    def First(self) -> False: ...
    def Last(self) -> False: ...
    def Append(self, theItem: False) -> False: ...
    def Prepend(self, theItem: False) -> False: ...
    def RemoveFirst(self) -> None: ...
    def Reverse(self) -> None: ...
    def Value(self, theIndex: int) -> False: ...
    def SetValue(self, theIndex: int, theValue: False) -> None: ...

class TDF_AttributeSequence:
    def __init__(self) -> None: ...
    def __len__(self) -> int: ...
    def Size(self) -> int: ...
    def Clear(self) -> None: ...
    def First(self) -> False: ...
    def Last(self) -> False: ...
    def Length(self) -> int: ...
    def Append(self, theItem: False) -> False: ...
    def Prepend(self, theItem: False) -> False: ...
    def RemoveFirst(self) -> None: ...
    def Reverse(self) -> None: ...
    def Value(self, theIndex: int) -> False: ...
    def SetValue(self, theIndex: int, theValue: False) -> None: ...

class TDF_DeltaList:
    def __init__(self) -> None: ...
    def __len__(self) -> int: ...
    def Size(self) -> int: ...
    def Clear(self) -> None: ...
    def First(self) -> False: ...
    def Last(self) -> False: ...
    def Append(self, theItem: False) -> False: ...
    def Prepend(self, theItem: False) -> False: ...
    def RemoveFirst(self) -> None: ...
    def Reverse(self) -> None: ...
    def Value(self, theIndex: int) -> False: ...
    def SetValue(self, theIndex: int, theValue: False) -> None: ...

class TDF_IDList:
    def __init__(self) -> None: ...
    def __len__(self) -> int: ...
    def Size(self) -> int: ...
    def Clear(self) -> None: ...
    def First(self) -> Standard_GUID: ...
    def Last(self) -> Standard_GUID: ...
    def Append(self, theItem: Standard_GUID) -> Standard_GUID: ...
    def Prepend(self, theItem: Standard_GUID) -> Standard_GUID: ...
    def RemoveFirst(self) -> None: ...
    def Reverse(self) -> None: ...
    def Value(self, theIndex: int) -> Standard_GUID: ...
    def SetValue(self, theIndex: int, theValue: Standard_GUID) -> None: ...

class TDF_LabelList:
    def __init__(self) -> None: ...
    def __len__(self) -> int: ...
    def Size(self) -> int: ...
    def Clear(self) -> None: ...
    def First(self) -> TDF_Label: ...
    def Last(self) -> TDF_Label: ...
    def Append(self, theItem: TDF_Label) -> TDF_Label: ...
    def Prepend(self, theItem: TDF_Label) -> TDF_Label: ...
    def RemoveFirst(self) -> None: ...
    def Reverse(self) -> None: ...
    def Value(self, theIndex: int) -> TDF_Label: ...
    def SetValue(self, theIndex: int, theValue: TDF_Label) -> None: ...

class TDF_LabelSequence:
    def __init__(self) -> None: ...
    def __len__(self) -> int: ...
    def Size(self) -> int: ...
    def Clear(self) -> None: ...
    def First(self) -> TDF_Label: ...
    def Last(self) -> TDF_Label: ...
    def Length(self) -> int: ...
    def Append(self, theItem: TDF_Label) -> TDF_Label: ...
    def Prepend(self, theItem: TDF_Label) -> TDF_Label: ...
    def RemoveFirst(self) -> None: ...
    def Reverse(self) -> None: ...
    def Value(self, theIndex: int) -> TDF_Label: ...
    def SetValue(self, theIndex: int, theValue: TDF_Label) -> None: ...

class tdf:
	@staticmethod
	def AddLinkGUIDToProgID(ID: Standard_GUID, ProgID: TCollection_ExtendedString) -> None: ...
	@staticmethod
	def GUIDFromProgID(ProgID: TCollection_ExtendedString, ID: Standard_GUID) -> bool: ...
	@staticmethod
	def LowestID() -> Standard_GUID: ...
	@staticmethod
	def ProgIDFromGUID(ID: Standard_GUID, ProgID: TCollection_ExtendedString) -> bool: ...
	@staticmethod
	def UppestID() -> Standard_GUID: ...

class TDF_Attribute(Standard_Transient):
	def AddAttribute(self, other: TDF_Attribute) -> None: ...
	def AfterAddition(self) -> None: ...
	def AfterResume(self) -> None: ...
	def AfterRetrieval(self, forceIt: Optional[bool] = False) -> bool: ...
	def AfterUndo(self, anAttDelta: TDF_AttributeDelta, forceIt: Optional[bool] = False) -> bool: ...
	def Backup(self) -> None: ...
	def BackupCopy(self) -> TDF_Attribute: ...
	def BeforeCommitTransaction(self) -> None: ...
	def BeforeForget(self) -> None: ...
	def BeforeRemoval(self) -> None: ...
	def BeforeUndo(self, anAttDelta: TDF_AttributeDelta, forceIt: Optional[bool] = False) -> bool: ...
	def DeltaOnAddition(self) -> TDF_DeltaOnAddition: ...
	def DeltaOnForget(self) -> TDF_DeltaOnForget: ...
	@overload
	def DeltaOnModification(self, anOldAttribute: TDF_Attribute) -> TDF_DeltaOnModification: ...
	@overload
	def DeltaOnModification(self, aDelta: TDF_DeltaOnModification) -> None: ...
	def DeltaOnRemoval(self) -> TDF_DeltaOnRemoval: ...
	def DeltaOnResume(self) -> TDF_DeltaOnResume: ...
	@overload
	def FindAttribute(self, anID: Standard_GUID, anAttribute: TDF_Attribute) -> bool: ...
	def Forget(self, aTransaction: int) -> None: ...
	def ForgetAllAttributes(self, clearChildren: Optional[bool] = True) -> None: ...
	def ForgetAttribute(self, aguid: Standard_GUID) -> bool: ...
	def ID(self) -> Standard_GUID: ...
	def IsAttribute(self, anID: Standard_GUID) -> bool: ...
	def IsBackuped(self) -> bool: ...
	def IsForgotten(self) -> bool: ...
	def IsNew(self) -> bool: ...
	def IsValid(self) -> bool: ...
	def Label(self) -> TDF_Label: ...
	def NewEmpty(self) -> TDF_Attribute: ...
	def Paste(self, intoAttribute: TDF_Attribute, aRelocationTable: TDF_RelocationTable) -> None: ...
	def References(self, aDataSet: TDF_DataSet) -> None: ...
	def Restore(self, anAttribute: TDF_Attribute) -> None: ...
	@overload
	def SetID(self) -> None: ...
	def Transaction(self) -> int: ...
	def UntilTransaction(self) -> int: ...

class TDF_AttributeDelta(Standard_Transient):
	def Apply(self) -> None: ...
	def Attribute(self) -> TDF_Attribute: ...
	def ID(self) -> Standard_GUID: ...
	def Label(self) -> TDF_Label: ...

class TDF_AttributeIterator:
	@overload
	def __init__(self) -> None: ...
	@overload
	def __init__(self, aLabel: TDF_Label, withoutForgotten: Optional[bool] = True) -> None: ...
	@overload
	def __init__(self, aLabelNode: TDF_LabelNodePtr, withoutForgotten: Optional[bool] = True) -> None: ...
	def Initialize(self, aLabel: TDF_Label, withoutForgotten: Optional[bool] = True) -> None: ...
	def More(self) -> bool: ...
	def Next(self) -> None: ...
	def PtrValue(self) -> TDF_Attribute: ...
	def Value(self) -> TDF_Attribute: ...

class TDF_ChildIDIterator:
	@overload
	def __init__(self) -> None: ...
	@overload
	def __init__(self, aLabel: TDF_Label, anID: Standard_GUID, allLevels: Optional[bool] = False) -> None: ...
	def Initialize(self, aLabel: TDF_Label, anID: Standard_GUID, allLevels: Optional[bool] = False) -> None: ...
	def More(self) -> bool: ...
	def Next(self) -> None: ...
	def NextBrother(self) -> None: ...
	def Value(self) -> TDF_Attribute: ...

class TDF_ChildIterator:
	@overload
	def __init__(self) -> None: ...
	@overload
	def __init__(self, aLabel: TDF_Label, allLevels: Optional[bool] = False) -> None: ...
	def Initialize(self, aLabel: TDF_Label, allLevels: Optional[bool] = False) -> None: ...
	def More(self) -> bool: ...
	def Next(self) -> None: ...
	def NextBrother(self) -> None: ...
	def Value(self) -> TDF_Label: ...

class TDF_ClosureMode:
	def __init__(self, aMode: Optional[bool] = True) -> None: ...
	@overload
	def Descendants(self, aStatus: bool) -> None: ...
	@overload
	def Descendants(self) -> bool: ...
	@overload
	def References(self, aStatus: bool) -> None: ...
	@overload
	def References(self) -> bool: ...

class TDF_ClosureTool:
	@overload
	@staticmethod
	def Closure(aDataSet: TDF_DataSet) -> None: ...
	@overload
	@staticmethod
	def Closure(aDataSet: TDF_DataSet, aFilter: TDF_IDFilter, aMode: TDF_ClosureMode) -> None: ...
	@overload
	@staticmethod
	def Closure(aLabel: TDF_Label, aLabMap: TDF_LabelMap, anAttMap: TDF_AttributeMap, aFilter: TDF_IDFilter, aMode: TDF_ClosureMode) -> None: ...

class TDF_ComparisonTool:
	@staticmethod
	def Compare(aSourceDataSet: TDF_DataSet, aTargetDataSet: TDF_DataSet, aFilter: TDF_IDFilter, aRelocationTable: TDF_RelocationTable) -> None: ...
	@staticmethod
	def Cut(aDataSet: TDF_DataSet) -> None: ...
	@staticmethod
	def IsSelfContained(aLabel: TDF_Label, aDataSet: TDF_DataSet) -> bool: ...
	@staticmethod
	def SourceUnbound(aRefDataSet: TDF_DataSet, aRelocationTable: TDF_RelocationTable, aFilter: TDF_IDFilter, aDiffDataSet: TDF_DataSet, anOption: Optional[int] = 2) -> bool: ...
	@staticmethod
	def TargetUnbound(aRefDataSet: TDF_DataSet, aRelocationTable: TDF_RelocationTable, aFilter: TDF_IDFilter, aDiffDataSet: TDF_DataSet, anOption: Optional[int] = 2) -> bool: ...

class TDF_CopyLabel:
	@overload
	def __init__(self) -> None: ...
	@overload
	def __init__(self, aSource: TDF_Label, aTarget: TDF_Label) -> None: ...
	@overload
	@staticmethod
	def ExternalReferences(Lab: TDF_Label, aExternals: TDF_AttributeMap, aFilter: TDF_IDFilter) -> bool: ...
	@overload
	@staticmethod
	def ExternalReferences(aRefLab: TDF_Label, Lab: TDF_Label, aExternals: TDF_AttributeMap, aFilter: TDF_IDFilter, aDataSet: TDF_DataSet) -> None: ...
	def IsDone(self) -> bool: ...
	def Load(self, aSource: TDF_Label, aTarget: TDF_Label) -> None: ...
	def Perform(self) -> None: ...
	def RelocationTable(self) -> TDF_RelocationTable: ...
	def UseFilter(self, aFilter: TDF_IDFilter) -> None: ...

class TDF_CopyTool:
	@overload
	@staticmethod
	def Copy(aSourceDataSet: TDF_DataSet, aRelocationTable: TDF_RelocationTable) -> None: ...
	@overload
	@staticmethod
	def Copy(aSourceDataSet: TDF_DataSet, aRelocationTable: TDF_RelocationTable, aPrivilegeFilter: TDF_IDFilter) -> None: ...
	@overload
	@staticmethod
	def Copy(aSourceDataSet: TDF_DataSet, aRelocationTable: TDF_RelocationTable, aPrivilegeFilter: TDF_IDFilter, aRefFilter: TDF_IDFilter, setSelfContained: bool) -> None: ...

class TDF_Data(Standard_Transient):
	def __init__(self) -> None: ...
	def AllowModification(self, isAllowed: bool) -> None: ...
	def Destroy(self) -> None: ...
	def IsApplicable(self, aDelta: TDF_Delta) -> bool: ...
	def IsModificationAllowed(self) -> bool: ...
	def LabelNodeAllocator(self) -> TDF_HAllocator: ...
	def NotUndoMode(self) -> bool: ...
	def Root(self) -> TDF_Label: ...
	def Time(self) -> int: ...
	def Transaction(self) -> int: ...
	def Undo(self, aDelta: TDF_Delta, withDelta: Optional[bool] = False) -> TDF_Delta: ...

class TDF_DataSet(Standard_Transient):
	def __init__(self) -> None: ...
	def AddAttribute(self, anAttribute: TDF_Attribute) -> None: ...
	def AddLabel(self, aLabel: TDF_Label) -> None: ...
	def AddRoot(self, aLabel: TDF_Label) -> None: ...
	def Attributes(self) -> TDF_AttributeMap: ...
	def Clear(self) -> None: ...
	def ContainsAttribute(self, anAttribute: TDF_Attribute) -> bool: ...
	def ContainsLabel(self, aLabel: TDF_Label) -> bool: ...
	def IsEmpty(self) -> bool: ...
	def Labels(self) -> TDF_LabelMap: ...
	def Roots(self) -> TDF_LabelList: ...

class TDF_Delta(Standard_Transient):
	def __init__(self) -> None: ...
	def AttributeDeltas(self) -> TDF_AttributeDeltaList: ...
	def BeginTime(self) -> int: ...
	def EndTime(self) -> int: ...
	def IsApplicable(self, aCurrentTime: int) -> bool: ...
	def IsEmpty(self) -> bool: ...
	def Labels(self, aLabelList: TDF_LabelList) -> None: ...
	def Name(self) -> TCollection_ExtendedString: ...
	def SetName(self, theName: TCollection_ExtendedString) -> None: ...

class TDF_IDFilter:
	def __init__(self, ignoreMode: Optional[bool] = True) -> None: ...
	def Assign(self, theFilter: TDF_IDFilter) -> None: ...
	def Copy(self, fromFilter: TDF_IDFilter) -> None: ...
	def IDList(self, anIDList: TDF_IDList) -> None: ...
	@overload
	def Ignore(self, anID: Standard_GUID) -> None: ...
	@overload
	def Ignore(self, anIDList: TDF_IDList) -> None: ...
	@overload
	def IgnoreAll(self, ignore: bool) -> None: ...
	@overload
	def IgnoreAll(self) -> bool: ...
	@overload
	def IsIgnored(self, anID: Standard_GUID) -> bool: ...
	@overload
	def IsIgnored(self, anAtt: TDF_Attribute) -> bool: ...
	@overload
	def IsKept(self, anID: Standard_GUID) -> bool: ...
	@overload
	def IsKept(self, anAtt: TDF_Attribute) -> bool: ...
	@overload
	def Keep(self, anID: Standard_GUID) -> None: ...
	@overload
	def Keep(self, anIDList: TDF_IDList) -> None: ...

class TDF_Label:
	def __init__(self) -> None: ...
	def AddAttribute(self, anAttribute: TDF_Attribute, append: Optional[bool] = True) -> None: ...
	def AttributesModified(self) -> bool: ...
	def Data(self) -> TDF_Data: ...
	def Depth(self) -> int: ...
	def Father(self) -> TDF_Label: ...
	@overload
	def FindAttribute(self, anID: Standard_GUID, anAttribute: TDF_Attribute) -> bool: ...
	@overload
	def FindAttribute(self, anID: Standard_GUID, aTransaction: int, anAttribute: TDF_Attribute) -> bool: ...
	def FindChild(self, aTag: int, create: Optional[bool] = True) -> TDF_Label: ...
	def ForgetAllAttributes(self, clearChildren: Optional[bool] = True) -> None: ...
	@overload
	def ForgetAttribute(self, anAttribute: TDF_Attribute) -> None: ...
	@overload
	def ForgetAttribute(self, aguid: Standard_GUID) -> bool: ...
	def HasAttribute(self) -> bool: ...
	def HasChild(self) -> bool: ...
	def HasGreaterNode(self, otherLabel: TDF_Label) -> bool: ...
	def HasLowerNode(self, otherLabel: TDF_Label) -> bool: ...
	def Imported(self, aStatus: bool) -> None: ...
	def IsAttribute(self, anID: Standard_GUID) -> bool: ...
	def IsDescendant(self, aLabel: TDF_Label) -> bool: ...
	def IsDifferent(self, aLabel: TDF_Label) -> bool: ...
	def IsEqual(self, aLabel: TDF_Label) -> bool: ...
	def IsImported(self) -> bool: ...
	def IsNull(self) -> bool: ...
	def IsRoot(self) -> bool: ...
	def MayBeModified(self) -> bool: ...
	def NbAttributes(self) -> int: ...
	def NbChildren(self) -> int: ...
	def NewChild(self) -> TDF_Label: ...
	def Nullify(self) -> None: ...
	def ResumeAttribute(self, anAttribute: TDF_Attribute) -> None: ...
	def Root(self) -> TDF_Label: ...
	def Tag(self) -> int: ...
	def Transaction(self) -> int: ...

class TDF_LabelMapHasher:
	@staticmethod
	def HashCode(theLabel: TDF_Label, theUpperBound: int) -> int: ...
	@staticmethod
	def IsEqual(aLab1: TDF_Label, aLab2: TDF_Label) -> bool: ...

class TDF_RelocationTable(Standard_Transient):
	def __init__(self, selfRelocate: Optional[bool] = False) -> None: ...
	@overload
	def AfterRelocate(self, afterRelocate: bool) -> None: ...
	@overload
	def AfterRelocate(self) -> bool: ...
	def AttributeTable(self) -> TDF_AttributeDataMap: ...
	def Clear(self) -> None: ...
	@overload
	def HasRelocation(self, aSourceLabel: TDF_Label, aTargetLabel: TDF_Label) -> bool: ...
	@overload
	def HasRelocation(self, aSourceAttribute: TDF_Attribute, aTargetAttribute: TDF_Attribute) -> bool: ...
	def HasTransientRelocation(self, aSourceTransient: Standard_Transient, aTargetTransient: Standard_Transient) -> bool: ...
	def LabelTable(self) -> TDF_LabelDataMap: ...
	@overload
	def SelfRelocate(self, selfRelocate: bool) -> None: ...
	@overload
	def SelfRelocate(self) -> bool: ...
	@overload
	def SetRelocation(self, aSourceLabel: TDF_Label, aTargetLabel: TDF_Label) -> None: ...
	@overload
	def SetRelocation(self, aSourceAttribute: TDF_Attribute, aTargetAttribute: TDF_Attribute) -> None: ...
	def SetTransientRelocation(self, aSourceTransient: Standard_Transient, aTargetTransient: Standard_Transient) -> None: ...
	def TargetAttributeMap(self, anAttributeMap: TDF_AttributeMap) -> None: ...
	def TargetLabelMap(self, aLabelMap: TDF_LabelMap) -> None: ...
	def TransientTable(self) -> TColStd_IndexedDataMapOfTransientTransient: ...

class TDF_Tool:
	@staticmethod
	def CountLabels(aLabelList: TDF_LabelList, aLabelMap: TDF_LabelIntegerMap) -> None: ...
	@staticmethod
	def DeductLabels(aLabelList: TDF_LabelList, aLabelMap: TDF_LabelIntegerMap) -> None: ...
	@staticmethod
	def Entry(aLabel: TDF_Label, anEntry: TCollection_AsciiString) -> None: ...
	@overload
	@staticmethod
	def IsSelfContained(aLabel: TDF_Label) -> bool: ...
	@overload
	@staticmethod
	def IsSelfContained(aLabel: TDF_Label, aFilter: TDF_IDFilter) -> bool: ...
	@overload
	@staticmethod
	def Label(aDF: TDF_Data, anEntry: TCollection_AsciiString, aLabel: TDF_Label, create: Optional[bool] = False) -> None: ...
	@overload
	@staticmethod
	def Label(aDF: TDF_Data, anEntry: str, aLabel: TDF_Label, create: Optional[bool] = False) -> None: ...
	@overload
	@staticmethod
	def Label(aDF: TDF_Data, aTagList: TColStd_ListOfInteger, aLabel: TDF_Label, create: Optional[bool] = False) -> None: ...
	@overload
	@staticmethod
	def NbAttributes(aLabel: TDF_Label) -> int: ...
	@overload
	@staticmethod
	def NbAttributes(aLabel: TDF_Label, aFilter: TDF_IDFilter) -> int: ...
	@staticmethod
	def NbLabels(aLabel: TDF_Label) -> int: ...
	@overload
	@staticmethod
	def OutReferences(aLabel: TDF_Label, atts: TDF_AttributeMap) -> None: ...
	@overload
	@staticmethod
	def OutReferences(aLabel: TDF_Label, aFilterForReferers: TDF_IDFilter, aFilterForReferences: TDF_IDFilter, atts: TDF_AttributeMap) -> None: ...
	@overload
	@staticmethod
	def OutReferers(theLabel: TDF_Label, theAtts: TDF_AttributeMap) -> None: ...
	@overload
	@staticmethod
	def OutReferers(aLabel: TDF_Label, aFilterForReferers: TDF_IDFilter, aFilterForReferences: TDF_IDFilter, atts: TDF_AttributeMap) -> None: ...
	@staticmethod
	def RelocateLabel(aSourceLabel: TDF_Label, fromRoot: TDF_Label, toRoot: TDF_Label, aTargetLabel: TDF_Label, create: Optional[bool] = False) -> None: ...
	@overload
	@staticmethod
	def TagList(aLabel: TDF_Label, aTagList: TColStd_ListOfInteger) -> None: ...
	@overload
	@staticmethod
	def TagList(anEntry: TCollection_AsciiString, aTagList: TColStd_ListOfInteger) -> None: ...

class TDF_Transaction:
	@overload
	def __init__(self, aName: Optional[TCollection_AsciiString] = "") -> None: ...
	@overload
	def __init__(self, aDF: TDF_Data, aName: Optional[TCollection_AsciiString] = "") -> None: ...
	def Abort(self) -> None: ...
	def Commit(self, withDelta: Optional[bool] = False) -> TDF_Delta: ...
	def Data(self) -> TDF_Data: ...
	def Initialize(self, aDF: TDF_Data) -> None: ...
	def IsOpen(self) -> bool: ...
	def Name(self) -> TCollection_AsciiString: ...
	def Open(self) -> int: ...
	def Transaction(self) -> int: ...

class TDF_DeltaOnAddition(TDF_AttributeDelta):
	def __init__(self, anAtt: TDF_Attribute) -> None: ...
	def Apply(self) -> None: ...

class TDF_DeltaOnForget(TDF_AttributeDelta):
	def __init__(self, anAtt: TDF_Attribute) -> None: ...
	def Apply(self) -> None: ...

class TDF_DeltaOnModification(TDF_AttributeDelta):
	def Apply(self) -> None: ...

class TDF_DeltaOnRemoval(TDF_AttributeDelta):
	pass

class TDF_DeltaOnResume(TDF_AttributeDelta):
	def __init__(self, anAtt: TDF_Attribute) -> None: ...
	def Apply(self) -> None: ...

class TDF_Reference(TDF_Attribute):
	def __init__(self) -> None: ...
	def Get(self) -> TDF_Label: ...
	@staticmethod
	def GetID() -> Standard_GUID: ...
	def ID(self) -> Standard_GUID: ...
	def NewEmpty(self) -> TDF_Attribute: ...
	def Paste(self, Into: TDF_Attribute, RT: TDF_RelocationTable) -> None: ...
	def References(self, DS: TDF_DataSet) -> None: ...
	def Restore(self, With: TDF_Attribute) -> None: ...
	@overload
	@staticmethod
	def Set(I: TDF_Label, Origin: TDF_Label) -> TDF_Reference: ...
	@overload
	def Set(self, Origin: TDF_Label) -> None: ...

class TDF_TagSource(TDF_Attribute):
	def __init__(self) -> None: ...
	def Get(self) -> int: ...
	@staticmethod
	def GetID() -> Standard_GUID: ...
	def ID(self) -> Standard_GUID: ...
	@overload
	@staticmethod
	def NewChild(L: TDF_Label) -> TDF_Label: ...
	@overload
	def NewChild(self) -> TDF_Label: ...
	def NewEmpty(self) -> TDF_Attribute: ...
	def NewTag(self) -> int: ...
	def Paste(self, Into: TDF_Attribute, RT: TDF_RelocationTable) -> None: ...
	def Restore(self, with_: TDF_Attribute) -> None: ...
	@overload
	@staticmethod
	def Set(label: TDF_Label) -> TDF_TagSource: ...
	@overload
	def Set(self, T: int) -> None: ...

class TDF_DefaultDeltaOnModification(TDF_DeltaOnModification):
	def __init__(self, anAttribute: TDF_Attribute) -> None: ...
	def Apply(self) -> None: ...

class TDF_DefaultDeltaOnRemoval(TDF_DeltaOnRemoval):
	def __init__(self, anAttribute: TDF_Attribute) -> None: ...
	def Apply(self) -> None: ...

#classnotwrapped
class TDF_LabelNode: ...

#classnotwrapped
class TDF_DerivedAttribute: ...

# harray1 classes

class TDF_HAttributeArray1(TDF_AttributeArray1, Standard_Transient):
    def __init__(self, theLower: int, theUpper: int) -> None: ...
    def Array1(self) -> TDF_AttributeArray1: ...

# harray2 classes
# hsequence classes

tdf_AddLinkGUIDToProgID = tdf.AddLinkGUIDToProgID
tdf_GUIDFromProgID = tdf.GUIDFromProgID
tdf_LowestID = tdf.LowestID
tdf_ProgIDFromGUID = tdf.ProgIDFromGUID
tdf_UppestID = tdf.UppestID
TDF_ClosureTool_Closure = TDF_ClosureTool.Closure
TDF_ClosureTool_Closure = TDF_ClosureTool.Closure
TDF_ClosureTool_Closure = TDF_ClosureTool.Closure
TDF_ComparisonTool_Compare = TDF_ComparisonTool.Compare
TDF_ComparisonTool_Cut = TDF_ComparisonTool.Cut
TDF_ComparisonTool_IsSelfContained = TDF_ComparisonTool.IsSelfContained
TDF_ComparisonTool_SourceUnbound = TDF_ComparisonTool.SourceUnbound
TDF_ComparisonTool_TargetUnbound = TDF_ComparisonTool.TargetUnbound
TDF_CopyLabel_ExternalReferences = TDF_CopyLabel.ExternalReferences
TDF_CopyLabel_ExternalReferences = TDF_CopyLabel.ExternalReferences
TDF_CopyTool_Copy = TDF_CopyTool.Copy
TDF_CopyTool_Copy = TDF_CopyTool.Copy
TDF_CopyTool_Copy = TDF_CopyTool.Copy
TDF_LabelMapHasher_HashCode = TDF_LabelMapHasher.HashCode
TDF_LabelMapHasher_IsEqual = TDF_LabelMapHasher.IsEqual
TDF_Tool_CountLabels = TDF_Tool.CountLabels
TDF_Tool_DeductLabels = TDF_Tool.DeductLabels
TDF_Tool_DeepDump = TDF_Tool.DeepDump
TDF_Tool_DeepDump = TDF_Tool.DeepDump
TDF_Tool_Entry = TDF_Tool.Entry
TDF_Tool_ExtendedDeepDump = TDF_Tool.ExtendedDeepDump
TDF_Tool_ExtendedDeepDump = TDF_Tool.ExtendedDeepDump
TDF_Tool_IsSelfContained = TDF_Tool.IsSelfContained
TDF_Tool_IsSelfContained = TDF_Tool.IsSelfContained
TDF_Tool_Label = TDF_Tool.Label
TDF_Tool_Label = TDF_Tool.Label
TDF_Tool_Label = TDF_Tool.Label
TDF_Tool_NbAttributes = TDF_Tool.NbAttributes
TDF_Tool_NbAttributes = TDF_Tool.NbAttributes
TDF_Tool_NbLabels = TDF_Tool.NbLabels
TDF_Tool_OutReferences = TDF_Tool.OutReferences
TDF_Tool_OutReferences = TDF_Tool.OutReferences
TDF_Tool_OutReferers = TDF_Tool.OutReferers
TDF_Tool_OutReferers = TDF_Tool.OutReferers
TDF_Tool_RelocateLabel = TDF_Tool.RelocateLabel
TDF_Tool_TagList = TDF_Tool.TagList
TDF_Tool_TagList = TDF_Tool.TagList
TDF_Reference_GetID = TDF_Reference.GetID
TDF_Reference_Set = TDF_Reference.Set
TDF_TagSource_GetID = TDF_TagSource.GetID
TDF_TagSource_NewChild = TDF_TagSource.NewChild
TDF_TagSource_Set = TDF_TagSource.Set



#!/usr/bin/env python

##Copyright 2008-2017 Thomas Paviot (tpaviot@gmail.com)
##
##This file is part of pythonOCC.
##
##pythonOCC is free software: you can redistribute it and/or modify
##it under the terms of the GNU Lesser General Public License as published by
##the Free Software Foundation, either version 3 of the License, or
##(at your option) any later version.
##
##pythonOCC is distributed in the hope that it will be useful,
##but WITHOUT ANY WARRANTY; without even the implied warranty of
##MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##GNU Lesser General Public License for more details.
##
##You should have received a copy of the GNU Lesser General Public License
##along with pythonOCC.  If not, see <http://www.gnu.org/licenses/>.

import itertools
import math
import os
import sys
import time

import OCC
from OCC.Core.Aspect import Aspect_GFM_VER
from OCC.Core.AIS import AIS_Shape, AIS_Shaded, AIS_TexturedShape, AIS_WireFrame, AIS_Shape_SelectionMode
from OCC.Core.gp import gp_Dir, gp_Pnt, gp_Pnt2d, gp_Vec
from OCC.Core.BRepBuilderAPI import (BRepBuilderAPI_MakeVertex,
                                     BRepBuilderAPI_MakeEdge,
                                     BRepBuilderAPI_MakeEdge2d,
                                     BRepBuilderAPI_MakeFace)
from OCC.Core.TopAbs import (TopAbs_FACE, TopAbs_EDGE, TopAbs_VERTEX,
                             TopAbs_SHELL, TopAbs_SOLID)
from OCC.Core.Geom import Geom_Curve, Geom_Surface
from OCC.Core.Geom2d import Geom2d_Curve
from OCC.Core.Visualization import Display3d
from OCC.Core.V3d import (V3d_ZBUFFER, V3d_Zpos, V3d_Zneg, V3d_Xpos,
                          V3d_Xneg, V3d_Ypos, V3d_Yneg, V3d_XposYnegZpos)
from OCC.Core.TCollection import TCollection_ExtendedString, TCollection_AsciiString
from OCC.Core.Quantity import (Quantity_Color, Quantity_TOC_RGB, Quantity_NOC_WHITE,
                               Quantity_NOC_BLACK, Quantity_NOC_BLUE1,
                               Quantity_NOC_CYAN1, Quantity_NOC_RED,
                               Quantity_NOC_GREEN, Quantity_NOC_ORANGE, Quantity_NOC_YELLOW)
from OCC.Core.Prs3d import Prs3d_Arrow, Prs3d_Text, Prs3d_TextAspect
from OCC.Core.Graphic3d import (Graphic3d_NOM_NEON_GNC, Graphic3d_NOT_ENV_CLOUDS,
                                Handle_Graphic3d_TextureEnv_Create, Graphic3d_TextureEnv,
                                Graphic3d_Camera, Graphic3d_RM_RAYTRACING,
                                Graphic3d_RM_RASTERIZATION,
                                Graphic3d_StereoMode_QuadBuffer,
                                Graphic3d_RenderingParams,
                                Graphic3d_MaterialAspect,
                                Graphic3d_TOSM_FRAGMENT,
                                Graphic3d_Structure
                                )
from OCC.Core.Aspect import (Aspect_TOTP_RIGHT_LOWER, Aspect_FM_STRETCH,
                             Aspect_FM_NONE)

# Shaders and Units definition must be found by occ
# the fastest way to get done is to set the CASROOT env variable
# it must point to the /share folder.
if sys.platform == "win32":
    # do the same for Units
    if "CASROOT" in os.environ:
        casroot_path = os.environ["CASROOT"]
        # raise an error, force the user to correctly set the variable
        err_msg = "Please set the CASROOT env variable (%s is not ok)" % casroot_path
        if not os.path.isdir(casroot_path):
            raise AssertionError(err_msg)
    else:  # on miniconda or anaconda or whatever conda
        occ_package_path = os.path.dirname(OCC.__file__)
        casroot_path = os.path.join(occ_package_path, '..', '..', '..',
                                    'Library', 'share', 'oce')
        # we check that all required files are at the right place
        shaders_dict_found = os.path.isdir(os.path.join(casroot_path,
                                                        'src', 'Shaders'))
        unitlexicon_found = os.path.isfile(os.path.join(casroot_path,
                                                        'src', 'UnitsAPI',
                                                        'Lexi_Expr.dat'))
        unitsdefinition_found = os.path.isfile(os.path.join(casroot_path,
                                                            'src', 'UnitsAPI',
                                                            'Units.dat'))
        if shaders_dict_found and unitlexicon_found and unitsdefinition_found:
            os.environ["CASROOT"] = casroot_path


def rgb_color(r, g, b):
    return Quantity_Color(r, g, b, Quantity_TOC_RGB)


def get_color_from_name(color_name):
    ''' from the string 'WHITE', returns Quantity_Color
    WHITE.
    color_name is the color name, case insensitive.
    '''
    enum_name = 'Quantity_NOC_%s' % color_name.upper()
    if enum_name in globals():
        color_num = globals()[enum_name]
    elif enum_name+'1' in globals():
        color_num = globals()[enum_name+'1']
        print('Many colors for color name %s, using first.' % color_name)
    else:
        color_num = Quantity_NOC_WHITE
        print('Color name not defined. Use White by default')
    return Quantity_Color(color_num)


def to_string(_string):
    return TCollection_ExtendedString(_string)

# some thing we'll need later
modes = itertools.cycle([TopAbs_FACE, TopAbs_EDGE,
                         TopAbs_VERTEX,
                         TopAbs_SHELL, TopAbs_SOLID])


class Viewer3d(Display3d):
    def __init__(self):
        Display3d.__init__(self)
        self._parent = None  # the parent opengl GUI container

        self._inited = False
        self._local_context_opened = False

        self.Context = self.GetContext()
        self.Viewer = self.GetViewer()
        self.View = self.GetView()

        self.OverLayer = None
        self.default_drawer = None
        self._struc_mgr = None
        self._is_offscreen = None

        self.selected_shapes = []
        self._select_callbacks = []
        self._overlay_items = []

    def get_parent(self):
        return self._parent

    def register_overlay_item(self, overlay_item):
        self._overlay_items.append(overlay_item)
        self.View.MustBeResized()
        self.View.Redraw()

    def GetOverLayer(self):
        """ returns an handle to the current overlayer
        """
        return self.OverLayer

    def register_select_callback(self, callback):
        """ Adds a callback that will be called each time a shape s selected
        """
        if not callable(callback):
            raise AssertionError("You must provide a callable to register the callback")
        self._select_callbacks.append(callback)

    def unregister_callback(self, callback):
        """ Remove a callback from the callback list
        """
        if not callback in self._select_callbacks:
            raise AssertionError("This callback is not registered")
        self._select_callbacks.remove(callback)

    def MoveTo(self, X, Y):
        self.Context.MoveTo(X, Y, self.View, True)

    def FitAll(self):
        self.View.ZFitAll()
        self.View.FitAll()

    def Create(self, window_handle=None, parent=None, create_default_lights=True, draw_face_boundaries=True, phong_shading=True):
        self._window_handle = window_handle
        self._parent = parent

        if self._window_handle is None:
            self.InitOffscreen(640, 480)
            self._is_offscreen = True
        else:
            self.Init(self._window_handle)
            self._is_offscreen = False

        if create_default_lights:
            self.Viewer.SetDefaultLights()
            self.Viewer.SetLightOn()

        self.camera = self.View.Camera()
        self.default_drawer = self.Context.DefaultDrawer()

        # draw black contour edges, like other famous CAD packages
        if draw_face_boundaries:
            self.default_drawer.SetFaceBoundaryDraw(True)

        # turn up tesselation defaults, which are too conversative...
        chord_dev = self.default_drawer.MaximalChordialDeviation() / 10.
        self.default_drawer.SetMaximalChordialDeviation(chord_dev)

        if phong_shading:
            # gouraud shading by default, prefer phong instead
            self.View.SetShadingModel(Graphic3d_TOSM_FRAGMENT)

        # the selected elements gray by default, better to use orange...
        # self.Context.SelectionColor(Quantity_NOC_ORANGE)

        # necessary for text rendering
        self._struc_mgr = self.Context.MainPrsMgr().StructureManager()

        # by default, enable antialisaing
        self.EnableAntiAliasing()

        # turn self._inited flag to True
        self._inited = True

    def OnResize(self):
        self.View.MustBeResized()

    def ResetView(self):
        self.View.Reset()

    def Repaint(self):
        self.Viewer.Redraw()

    def SetModeWireFrame(self):
        self.View.SetComputedMode(False)
        self.Context.SetDisplayMode(AIS_WireFrame, True)

    def SetModeShaded(self):
        self.View.SetComputedMode(False)
        self.Context.SetDisplayMode(AIS_Shaded, True)

    def SetModeHLR(self):
        self.View.SetComputedMode(True)

    def SetOrthographicProjection(self):
        self.camera.SetProjectionType(Graphic3d_Camera.Projection_Orthographic)

    def SetPerspectiveProjection(self):
        self.camera.SetProjectionType(Graphic3d_Camera.Projection_Perspective)

    def View_Top(self):
        self.View.SetProj(V3d_Zpos)

    def View_Bottom(self):
        self.View.SetProj(V3d_Zneg)

    def View_Left(self):
        self.View.SetProj(V3d_Xneg)

    def View_Right(self):
        self.View.SetProj(V3d_Xpos)

    def View_Front(self):
        self.View.SetProj(V3d_Yneg)

    def View_Rear(self):
        self.View.SetProj(V3d_Ypos)

    def View_Iso(self):
        self.View.SetProj(V3d_XposYnegZpos)

    def EnableTextureEnv(self, name_of_texture=Graphic3d_NOT_ENV_CLOUDS):
        """ enable environment mapping. Possible modes are
        Graphic3d_NOT_ENV_CLOUDS
        Graphic3d_NOT_ENV_CV
        Graphic3d_NOT_ENV_MEDIT
        Graphic3d_NOT_ENV_PEARL
        Graphic3d_NOT_ENV_SKY1
        Graphic3d_NOT_ENV_SKY2
        Graphic3d_NOT_ENV_LINES
        Graphic3d_NOT_ENV_ROAD
        Graphic3d_NOT_ENV_UNKNOWN
        """
        texture_env = Graphic3d_TextureEnv(name_of_texture)
        self.View.SetTextureEnv(texture_env)
        self.View.Redraw()

    def DisableTextureEnv(self):
        a_null_texture = Handle_Graphic3d_TextureEnv_Create()
        self.View.SetTextureEnv(a_null_texture) # Passing null handle to clear the texture data
        self.View.Redraw()

    def SetRenderingParams(self,
                           Method=Graphic3d_RM_RASTERIZATION,
                           RaytracingDepth=3,
                           IsShadowEnabled=True,
                           IsReflectionEnabled=False,
                           IsAntialiasingEnabled=False,
                           IsTransparentShadowEnabled=False,
                           StereoMode=Graphic3d_StereoMode_QuadBuffer,
                           AnaglyphFilter=Graphic3d_RenderingParams.Anaglyph_RedCyan_Optimized,
                           ToReverseStereo=False):
        """ Default values are :
            Method=Graphic3d_RM_RASTERIZATION,
            RaytracingDepth=3,
            IsShadowEnabled=True,
            IsReflectionEnabled=False,
            IsAntialiasingEnabled=False,
            IsTransparentShadowEnabled=False,
            StereoMode=Graphic3d_StereoMode_QuadBuffer,
            AnaglyphFilter=Graphic3d_RenderingParams.Anaglyph_RedCyan_Optimized,
            ToReverseStereo=False)
        """
        self.ChangeRenderingParams(Method,
                                   RaytracingDepth,
                                   IsShadowEnabled,
                                   IsReflectionEnabled,
                                   IsAntialiasingEnabled,
                                   IsTransparentShadowEnabled,
                                   StereoMode,
                                   AnaglyphFilter,
                                   ToReverseStereo)

    def SetRasterizationMode(self):
        """ to enable rasterization mode, just call the SetRenderingParams
        with default values
        """
        self.SetRenderingParams()

    def SetRaytracingMode(self, depth=3):
        """ enables the raytracing mode
        """
        self.SetRenderingParams(Method=Graphic3d_RM_RAYTRACING,
                                RaytracingDepth=depth,
                                IsAntialiasingEnabled=True,
                                IsShadowEnabled=True,
                                IsReflectionEnabled=True,
                                IsTransparentShadowEnabled=True)

    def ExportToImage(self, image_filename):
        self.View.Dump(image_filename)

    def display_graduated_trihedron(self):
        self.View.GraduatedTrihedronDisplay()

    def display_triedron(self):
        """ Show a black triedron in lower right corner
        """
        self.View.TriedronDisplay(Aspect_TOTP_RIGHT_LOWER, Quantity_Color(Quantity_NOC_BLACK), 0.1, V3d_ZBUFFER)

    def hide_triedron(self):
        """ Show a black triedron in lower right corner
        """
        self.View.TriedronErase()

    def set_bg_gradient_color(self, color1, color2, fill_method=Aspect_GFM_VER):
        """ set a bg vertical gradient color.
        color1 is [R1, G1, B1], each being bytes or an instance of Quantity_Color
        color2 is [R2, G2, B2], each being bytes or an instance of Quantity_Color
        fill_method is one of Aspect_GFM_VER value Aspect_GFM_NONE, Aspect_GFM_HOR,
        Aspect_GFM_VER, Aspect_GFM_DIAG1, Aspect_GFM_DIAG2, Aspect_GFM_CORNER1, Aspect_GFM_CORNER2,
        Aspect_GFM_CORNER3, Aspect_GFM_CORNER4
        """
        if isinstance(color1, list) and isinstance(color2, list):
            R1, G1, B1 = color1
            R2, G2, B2 = color2
            color1 = rgb_color(float(R1)/255., float(G1)/255., float(B1)/255.)
            color2 = rgb_color(float(R2)/255., float(G2)/255., float(B2)/255.)
        elif not isinstance(color1, Quantity_Color) and isinstance(color2, Quantity_Color):
            raise AssertionError("color1 and color2 mmust be either [R, G, B] lists or a Quantity_Color")
        self.View.SetBgGradientColors(color1, color2, fill_method, True)

    def SetBackgroundImage(self, image_filename, stretch=True):
        """ displays a background image (jpg, png etc.)
        """
        if not os.path.isfile(image_filename):
            raise IOError("image file %s not found." % image_filename)
        if stretch:
            self.View.SetBackgroundImage(image_filename, Aspect_FM_STRETCH, True)
        else:
            self.View.SetBackgroundImage(image_filename, Aspect_FM_NONE, True)

    def DisplayVector(self, vec, pnt, update=False):
        """ displays a vector as an arrow
        """
        if self._inited:
            aStructure = Graphic3d_Structure(self._struc_mgr)

            pnt_as_vec = gp_Vec(pnt.X(), pnt.Y(), pnt.Z())
            start = pnt_as_vec + vec
            pnt_start = gp_Pnt(start.X(), start.Y(), start.Z())

            Prs3d_Arrow.Draw(
                aStructure,
                pnt_start,
                gp_Dir(vec),
                math.radians(20),
                vec.Magnitude()
            )
            aStructure.Display()
            # it would be more coherent if a AIS_InteractiveObject
            # would be returned
            if update:
                self.Repaint()
            return aStructure

    def DisplayMessage(self, point, text_to_write, height=None, message_color=None, update=False):
        """
        :point: a gp_Pnt or gp_Pnt2d instance
        :text_to_write: a string
        :message_color: triple with the range 0-1
        """
        aStructure = Graphic3d_Structure(self._struc_mgr)
        text_aspect = Prs3d_TextAspect()

        if message_color is not None:
            text_aspect.SetColor(rgb_color(*message_color))
        if height is not None:
            text_aspect.SetHeight(height)
        if isinstance(point, gp_Pnt2d):
            point = gp_Pnt(point.X(), point.Y(), 0)
        Prs3d_Text.Draw(aStructure,
                        text_aspect,
                        to_string(text_to_write),
                        point)
        aStructure.Display()
        # @TODO: it would be more coherent if a AIS_InteractiveObject
        # is be returned
        if update:
            self.Repaint()
        return aStructure

    def DisplayShape(self, shapes, material=None, texture=None, color=None, transparency=None, update=False):
        """ display one or a set of displayable objects
        """
        ais_shapes = []  # the list of all displayed shapes

        if issubclass(shapes.__class__, gp_Pnt):
            # if a gp_Pnt is passed, first convert to vertex
            vertex = BRepBuilderAPI_MakeVertex(shapes)
            shapes = [vertex.Shape()]
        elif isinstance(shapes, gp_Pnt2d):
            vertex = BRepBuilderAPI_MakeVertex(gp_Pnt(shapes.X(), shapes.Y(), 0))
            shapes = [vertex.Shape()]
        elif isinstance(shapes, Geom_Surface):
            bounds = True
            toldegen = 1e-6
            face = BRepBuilderAPI_MakeFace()
            face.Init(shapes, bounds, toldegen)
            face.Build()
            shapes = [face.Shape()]
        elif isinstance(shapes, Geom_Curve):
            edge = BRepBuilderAPI_MakeEdge(shapes)
            shapes = [edge.Shape()]
        elif isinstance(shapes, Geom2d_Curve):
            edge2d = BRepBuilderAPI_MakeEdge2d(shapes)
            shapes = [edge2d.Shape()]
        # if only one shapes, create a list with a single shape
        if not isinstance(shapes, list):
            shapes = [shapes]
        # build AIS_Shapes list
        for shape in shapes:
            if material or texture:
                if texture:
                    shape_to_display = AIS_TexturedShape(shape)
                    filename, toScaleU, toScaleV, toRepeatU, toRepeatV, originU, originV = texture.GetProperties()
                    shape_to_display.SetTextureFileName(TCollection_AsciiString(filename))
                    shape_to_display.SetTextureMapOn()
                    shape_to_display.SetTextureScale(True, toScaleU, toScaleV)
                    shape_to_display.SetTextureRepeat(True, toRepeatU, toRepeatV)
                    shape_to_display.SetTextureOrigin(True, originU, originV)
                    shape_to_display.SetDisplayMode(3)
                elif material:
                    shape_to_display = AIS_Shape(shape)
                    shape_to_display.SetMaterial(Graphic3d_MaterialAspect(material))
            else:
                # TODO: can we use .Set to attach all TopoDS_Shapes
                # to this AIS_Shape instance?
                shape_to_display = AIS_Shape(shape)

            ais_shapes.append(shape_to_display)

        # if not SOLO:
        #     # computing graphic properties is expensive
        #     # if an iterable is found, so cluster all TopoDS_Shape under
        #     # an AIS_MultipleConnectedInteractive
        #     #shape_to_display = AIS_MultipleConnectedInteractive()
        #     for ais_shp in ais_shapes:
        #         # TODO : following line crashes with oce-0.18
        #         # why ? fix ?
        #         #shape_to_display.Connect(i)
        #         self.Context.Display(ais_shp, False)
        # set the graphic properties
        if material is None:
            #The default material is too shiny to show the object
            #color well, so I set it to something less reflective
            for shape_to_display in ais_shapes:
                shape_to_display.SetMaterial(Graphic3d_MaterialAspect(Graphic3d_NOM_NEON_GNC))
        if color:
            if isinstance(color, str):
                color = get_color_from_name(color)
            elif isinstance(color, int):
                color = Quantity_Color(color)
            for shp in ais_shapes:
                self.Context.SetColor(shp, color, False)
        if transparency:
            for shape_to_display in ais_shapes:
                shape_to_display.SetTransparency(transparency)
        # display the shapes
        for shape_to_display in ais_shapes:
            self.Context.Display(shape_to_display, False)
        if update:
            # especially this call takes up a lot of time...
            self.FitAll()
            self.Repaint()

        return ais_shapes

    def DisplayColoredShape(self, shapes, color='YELLOW', update=False, ):
        if isinstance(color, str):
            dict_color = {'WHITE': Quantity_NOC_WHITE,
                          'BLUE': Quantity_NOC_BLUE1,
                          'RED': Quantity_NOC_RED,
                          'GREEN': Quantity_NOC_GREEN,
                          'YELLOW': Quantity_NOC_YELLOW,
                          'CYAN': Quantity_NOC_CYAN1,
                          'BLACK': Quantity_NOC_BLACK,
                          'ORANGE': Quantity_NOC_ORANGE}
            clr = dict_color[color]
        elif isinstance(color, Quantity_Color):
            clr = color
        else:
            raise ValueError('color should either be a string ( "BLUE" ) or a Quantity_Color(0.1, 0.8, 0.1) got %s' % color)

        return self.DisplayShape(shapes, color=clr, update=update)

    def EnableAntiAliasing(self):
        self.SetNbMsaaSample(4)

    def DisableAntiAliasing(self):
        self.SetNbMsaaSample(0)

    def EraseAll(self):
        # nessecary to remove text added by DisplayMessage
        self.Context.PurgeDisplay()
        self.Context.EraseAll(True)

    def Tumble(self, num_images, animation=True):
        self.View.Tumble(num_images, animation)

    def Pan(self, dx, dy):
        self.View.Pan(dx, dy)

    def SetSelectionMode(self, mode=None):
        topo_level = next(modes)
        if mode is None:
            self.Context.Activate(AIS_Shape_SelectionMode(topo_level), True)
        else:
            self.Context.Activate(AIS_Shape_SelectionMode(mode), True)
        self.Context.UpdateSelected(True)

    def SetSelectionModeVertex(self):
        self.SetSelectionMode(TopAbs_VERTEX)

    def SetSelectionModeEdge(self):
        self.SetSelectionMode(TopAbs_EDGE)

    def SetSelectionModeFace(self):
        self.SetSelectionMode(TopAbs_FACE)

    def SetSelectionModeShape(self):
        self.Context.Deactivate()

    def SetSelectionModeNeutral(self):
        self.Context.Deactivate()

    def GetSelectedShapes(self):
        return self.selected_shapes

    def GetSelectedShape(self):
        """
        Returns the current selected shape
        """
        return self.selected_shape

    def SelectArea(self, Xmin, Ymin, Xmax, Ymax):
        self.Context.Select(Xmin, Ymin, Xmax, Ymax, self.View, True)
        self.Context.InitSelected()
        # reinit the selected_shapes list
        self.selected_shapes = []
        while self.Context.MoreSelected():
            if self.Context.HasSelectedShape():
                self.selected_shapes.append(self.Context.SelectedShape())
            self.Context.NextSelected()
        # callbacks
        for callback in self._select_callbacks:
            callback(self.selected_shapes, Xmin, Ymin, Xmax, Ymax)

    def Select(self, X, Y):
        self.Context.Select(True)
        self.Context.InitSelected()

        self.selected_shapes = []
        if self.Context.MoreSelected():
            if self.Context.HasSelectedShape():
                self.selected_shapes.append(self.Context.SelectedShape())
        # callbacks
        for callback in self._select_callbacks:
            callback(self.selected_shapes, X, Y)

    def ShiftSelect(self, X, Y):
        self.Context.ShiftSelect(True)
        self.Context.InitSelected()

        self.selected_shapes = []
        while self.Context.MoreSelected():
            if self.Context.HasSelectedShape():
                self.selected_shapes.append(self.Context.SelectedShape())
            self.Context.NextSelected()
        # hilight newly selected unhighlight those no longer selected
        self.Context.UpdateSelected(True)
        # callbacks
        for callback in self._select_callbacks:
            callback(self.selected_shapes, X, Y)

    def Rotation(self, X, Y):
        self.View.Rotation(X, Y)

    def DynamicZoom(self, X1, Y1, X2, Y2):
        self.View.Zoom(X1, Y1, X2, Y2)

    def ZoomFactor(self, zoom_factor):
        self.View.SetZoom(zoom_factor)

    def ZoomArea(self, X1, Y1, X2, Y2):
        self.View.WindowFit(X1, Y1, X2, Y2)

    def Zoom(self, X, Y):
        self.View.Zoom(X, Y)

    def StartRotation(self, X, Y):
        self.View.StartRotation(X, Y)


class OffscreenRenderer(Viewer3d):
    """ The offscreen renderer is inherited from Viewer3d.
    The DisplayShape method is overriden to export to image
    each time it is called.
    """
    def __init__(self, screen_size=(640, 480)):
        Viewer3d.__init__(self)
        # create the renderer
        self.Create()
        self.SetSize(screen_size[0], screen_size[1])
        self.SetModeShaded()
        self.set_bg_gradient_color([206, 215, 222], [128, 128, 128])
        self.display_triedron()
        self.capture_number = 0

    def DisplayShape(self, shapes, material=None, texture=None, color=None, transparency=None, update=True):
        # call the "original" DisplayShape method
        r = super(OffscreenRenderer, self).DisplayShape(shapes, material, texture,
                                                        color, transparency, update)  # always update
        if os.getenv("PYTHONOCC_OFFSCREEN_RENDERER_DUMP_IMAGE") == "1":  # dump to jpeg file
            timestamp = ("%f" % time.time()).split(".")[0]
            self.capture_number += 1
            image_filename = "capture-%i-%s.jpeg" % (self.capture_number,
                                                     timestamp.replace(" ", "-"))
            if os.getenv("PYTHONOCC_OFFSCREEN_RENDERER_DUMP_IMAGE_PATH"):
                path = os.getenv("PYTHONOCC_OFFSCREEN_RENDERER_DUMP_IMAGE_PATH")
                if not os.path.isdir(path):
                    raise IOError("%s is not a valid path" % path)
            else:
                path = os.getcwd()
            image_full_name = os.path.join(path, image_filename)
            self.View.Dump(image_full_name)
            if not os.path.isfile(image_full_name):
                raise IOError("OffscreenRenderer failed to render image to file")
            print("OffscreenRenderer content dumped to %s" % image_full_name)
        return r



# This file was automatically generated by SWIG (http://www.swig.org).
# Version 4.0.2
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.

"""
BRepMeshData module, see official documentation at
https://www.opencascade.com/doc/occt-7.4.0/refman/html/package_brepmeshdata.html
"""

from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

# Import the low-level C/C++ module
if __package__ or "." in __name__:
    from . import _BRepMeshData
else:
    import _BRepMeshData

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)


class SwigPyIterator(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined - class is abstract")
    __repr__ = _swig_repr
    __swig_destroy__ = _BRepMeshData.delete_SwigPyIterator

    def value(self) -> "PyObject *":
        return _BRepMeshData.SwigPyIterator_value(self)

    def incr(self, n: "size_t"=1) -> "swig::SwigPyIterator *":
        return _BRepMeshData.SwigPyIterator_incr(self, n)

    def decr(self, n: "size_t"=1) -> "swig::SwigPyIterator *":
        return _BRepMeshData.SwigPyIterator_decr(self, n)

    def distance(self, x: "SwigPyIterator") -> "ptrdiff_t":
        return _BRepMeshData.SwigPyIterator_distance(self, x)

    def equal(self, x: "SwigPyIterator") -> "bool":
        return _BRepMeshData.SwigPyIterator_equal(self, x)

    def copy(self) -> "swig::SwigPyIterator *":
        return _BRepMeshData.SwigPyIterator_copy(self)

    def next(self) -> "PyObject *":
        return _BRepMeshData.SwigPyIterator_next(self)

    def __next__(self) -> "PyObject *":
        return _BRepMeshData.SwigPyIterator___next__(self)

    def previous(self) -> "PyObject *":
        return _BRepMeshData.SwigPyIterator_previous(self)

    def advance(self, n: "ptrdiff_t") -> "swig::SwigPyIterator *":
        return _BRepMeshData.SwigPyIterator_advance(self, n)

    def __eq__(self, x: "SwigPyIterator") -> "bool":
        return _BRepMeshData.SwigPyIterator___eq__(self, x)

    def __ne__(self, x: "SwigPyIterator") -> "bool":
        return _BRepMeshData.SwigPyIterator___ne__(self, x)

    def __iadd__(self, n: "ptrdiff_t") -> "swig::SwigPyIterator &":
        return _BRepMeshData.SwigPyIterator___iadd__(self, n)

    def __isub__(self, n: "ptrdiff_t") -> "swig::SwigPyIterator &":
        return _BRepMeshData.SwigPyIterator___isub__(self, n)

    def __add__(self, n: "ptrdiff_t") -> "swig::SwigPyIterator *":
        return _BRepMeshData.SwigPyIterator___add__(self, n)

    def __sub__(self, *args) -> "ptrdiff_t":
        return _BRepMeshData.SwigPyIterator___sub__(self, *args)
    def __iter__(self):
        return self

# Register SwigPyIterator in _BRepMeshData:
_BRepMeshData.SwigPyIterator_swigregister(SwigPyIterator)


def _dumps_object(klass):
    """ Overwrite default string output for any wrapped object.
    By default, __repr__ method returns something like:
    <OCC.Core.TopoDS.TopoDS_Shape; proxy of <Swig Object of type 'TopoDS_Shape *' at 0x02BB0758> >
    This is too much verbose.
    We prefer :
    <class 'gp_Pnt'>
    or
    <class 'TopoDS_Shape'>
    """
    klass_name = str(klass.__class__).split(".")[3].split("'")[0]
    repr_string = "<class '" + klass_name + "'"
# for TopoDS_Shape, we also look for the base type
    if klass_name == "TopoDS_Shape":
        if klass.IsNull():
            repr_string += ": Null>"
            return repr_string
        st = klass.ShapeType()
        types = {OCC.Core.TopAbs.TopAbs_VERTEX: "Vertex",
                 OCC.Core.TopAbs.TopAbs_SOLID: "Solid",
                 OCC.Core.TopAbs.TopAbs_EDGE: "Edge",
                 OCC.Core.TopAbs.TopAbs_FACE: "Face",
                 OCC.Core.TopAbs.TopAbs_SHELL: "Shell",
                 OCC.Core.TopAbs.TopAbs_WIRE: "Wire",
                 OCC.Core.TopAbs.TopAbs_COMPOUND: "Compound",
                 OCC.Core.TopAbs.TopAbs_COMPSOLID: "Compsolid"}
        repr_string += "; Type:%s" % types[st]        
    elif hasattr(klass, "IsNull"):
        if klass.IsNull():
            repr_string += "; Null"
    repr_string += ">"
    return repr_string


def process_exception(error: "Standard_Failure", method_name: "std::string", class_name: "std::string") -> "void":
    return _BRepMeshData.process_exception(error, method_name, class_name)

from six import with_metaclass
import warnings
from OCC.Wrapper.wrapper_utils import Proxy, deprecated

import OCC.Core.Standard
import OCC.Core.NCollection
import OCC.Core.IMeshData
import OCC.Core.TopoDS
import OCC.Core.Message
import OCC.Core.TCollection
import OCC.Core.OSD
import OCC.Core.TColStd
import OCC.Core.TopAbs
import OCC.Core.TopLoc
import OCC.Core.gp
import OCC.Core.BRepAdaptor
import OCC.Core.Adaptor3d
import OCC.Core.Geom
import OCC.Core.GeomAbs
import OCC.Core.TColgp
import OCC.Core.Adaptor2d
import OCC.Core.Geom2d
import OCC.Core.math
import OCC.Core.GeomAdaptor
import OCC.Core.Geom2dAdaptor

from enum import IntEnum
from OCC.Core.Exception import *



class BRepMeshData_Curve(OCC.Core.IMeshData.IMeshData_Curve):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined")
    __repr__ = _swig_repr

    __repr__ = _dumps_object

    @methodnotwrapped
    def B(self):
    	pass

    @methodnotwrapped
    def R(self):
    	pass

    @methodnotwrapped
    def e(self):
    	pass

    @methodnotwrapped
    def p(self):
    	pass

    @methodnotwrapped
    def M(self):
    	pass

    @methodnotwrapped
    def e(self):
    	pass

    @methodnotwrapped
    def s(self):
    	pass

    @methodnotwrapped
    def h(self):
    	pass

    @methodnotwrapped
    def D(self):
    	pass

    @methodnotwrapped
    def a(self):
    	pass

    @methodnotwrapped
    def t(self):
    	pass

    @methodnotwrapped
    def a(self):
    	pass

    @methodnotwrapped
    def _(self):
    	pass

    @methodnotwrapped
    def C(self):
    	pass

    @methodnotwrapped
    def u(self):
    	pass

    @methodnotwrapped
    def r(self):
    	pass

    @methodnotwrapped
    def v(self):
    	pass

    @methodnotwrapped
    def e(self):
    	pass

    __swig_destroy__ = _BRepMeshData.delete_BRepMeshData_Curve

# Register BRepMeshData_Curve in _BRepMeshData:
_BRepMeshData.BRepMeshData_Curve_swigregister(BRepMeshData_Curve)

class BRepMeshData_Edge(OCC.Core.IMeshData.IMeshData_Edge):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined")
    __repr__ = _swig_repr

    def GetPCurve(self, *args) -> "IMeshData::IPCurveHandle const &":
        r"""
        Returns pcurve for the specified discrete face.

        Parameters
        ----------
        theDFace: IMeshData::IFacePtr
        theOrientation: TopAbs_Orientation

        Returns
        -------
        IMeshData::IPCurveHandle
        Returns pcurve with the given index.

        Parameters
        ----------
        theIndex: int

        Returns
        -------
        IMeshData::IPCurveHandle

        """
        return _BRepMeshData.BRepMeshData_Edge_GetPCurve(self, *args)

    __repr__ = _dumps_object

    @methodnotwrapped
    def B(self):
    	pass

    @methodnotwrapped
    def R(self):
    	pass

    @methodnotwrapped
    def e(self):
    	pass

    @methodnotwrapped
    def p(self):
    	pass

    @methodnotwrapped
    def M(self):
    	pass

    @methodnotwrapped
    def e(self):
    	pass

    @methodnotwrapped
    def s(self):
    	pass

    @methodnotwrapped
    def h(self):
    	pass

    @methodnotwrapped
    def D(self):
    	pass

    @methodnotwrapped
    def a(self):
    	pass

    @methodnotwrapped
    def t(self):
    	pass

    @methodnotwrapped
    def a(self):
    	pass

    @methodnotwrapped
    def _(self):
    	pass

    @methodnotwrapped
    def E(self):
    	pass

    @methodnotwrapped
    def d(self):
    	pass

    @methodnotwrapped
    def g(self):
    	pass

    @methodnotwrapped
    def e(self):
    	pass

    __swig_destroy__ = _BRepMeshData.delete_BRepMeshData_Edge

# Register BRepMeshData_Edge in _BRepMeshData:
_BRepMeshData.BRepMeshData_Edge_swigregister(BRepMeshData_Edge)

class BRepMeshData_Face(OCC.Core.IMeshData.IMeshData_Face):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined")
    __repr__ = _swig_repr

    __repr__ = _dumps_object

    @methodnotwrapped
    def B(self):
    	pass

    @methodnotwrapped
    def R(self):
    	pass

    @methodnotwrapped
    def e(self):
    	pass

    @methodnotwrapped
    def p(self):
    	pass

    @methodnotwrapped
    def M(self):
    	pass

    @methodnotwrapped
    def e(self):
    	pass

    @methodnotwrapped
    def s(self):
    	pass

    @methodnotwrapped
    def h(self):
    	pass

    @methodnotwrapped
    def D(self):
    	pass

    @methodnotwrapped
    def a(self):
    	pass

    @methodnotwrapped
    def t(self):
    	pass

    @methodnotwrapped
    def a(self):
    	pass

    @methodnotwrapped
    def _(self):
    	pass

    @methodnotwrapped
    def F(self):
    	pass

    @methodnotwrapped
    def a(self):
    	pass

    @methodnotwrapped
    def c(self):
    	pass

    @methodnotwrapped
    def e(self):
    	pass

    __swig_destroy__ = _BRepMeshData.delete_BRepMeshData_Face

# Register BRepMeshData_Face in _BRepMeshData:
_BRepMeshData.BRepMeshData_Face_swigregister(BRepMeshData_Face)

class BRepMeshData_Model(OCC.Core.IMeshData.IMeshData_Model):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        Constructor. initializes empty model.

        Parameters
        ----------
        theShape: TopoDS_Shape

        Returns
        -------
        None

        """
        _BRepMeshData.BRepMeshData_Model_swiginit(self, _BRepMeshData.new_BRepMeshData_Model(*args))

    def SetMaxSize(self, *args) -> "void":
        r"""
        Sets maximum size of shape's bounding box.

        Parameters
        ----------
        theValue: float

        Returns
        -------
        None

        """
        return _BRepMeshData.BRepMeshData_Model_SetMaxSize(self, *args)

    __repr__ = _dumps_object

    __swig_destroy__ = _BRepMeshData.delete_BRepMeshData_Model

# Register BRepMeshData_Model in _BRepMeshData:
_BRepMeshData.BRepMeshData_Model_swigregister(BRepMeshData_Model)

class BRepMeshData_PCurve(OCC.Core.IMeshData.IMeshData_PCurve):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined")
    __repr__ = _swig_repr

    __repr__ = _dumps_object

    @methodnotwrapped
    def B(self):
    	pass

    @methodnotwrapped
    def R(self):
    	pass

    @methodnotwrapped
    def e(self):
    	pass

    @methodnotwrapped
    def p(self):
    	pass

    @methodnotwrapped
    def M(self):
    	pass

    @methodnotwrapped
    def e(self):
    	pass

    @methodnotwrapped
    def s(self):
    	pass

    @methodnotwrapped
    def h(self):
    	pass

    @methodnotwrapped
    def D(self):
    	pass

    @methodnotwrapped
    def a(self):
    	pass

    @methodnotwrapped
    def t(self):
    	pass

    @methodnotwrapped
    def a(self):
    	pass

    @methodnotwrapped
    def _(self):
    	pass

    @methodnotwrapped
    def P(self):
    	pass

    @methodnotwrapped
    def C(self):
    	pass

    @methodnotwrapped
    def u(self):
    	pass

    @methodnotwrapped
    def r(self):
    	pass

    @methodnotwrapped
    def v(self):
    	pass

    @methodnotwrapped
    def e(self):
    	pass

    __swig_destroy__ = _BRepMeshData.delete_BRepMeshData_PCurve

# Register BRepMeshData_PCurve in _BRepMeshData:
_BRepMeshData.BRepMeshData_PCurve_swigregister(BRepMeshData_PCurve)

class BRepMeshData_Wire(OCC.Core.IMeshData.IMeshData_Wire):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined")
    __repr__ = _swig_repr

    __repr__ = _dumps_object

    @methodnotwrapped
    def B(self):
    	pass

    @methodnotwrapped
    def R(self):
    	pass

    @methodnotwrapped
    def e(self):
    	pass

    @methodnotwrapped
    def p(self):
    	pass

    @methodnotwrapped
    def M(self):
    	pass

    @methodnotwrapped
    def e(self):
    	pass

    @methodnotwrapped
    def s(self):
    	pass

    @methodnotwrapped
    def h(self):
    	pass

    @methodnotwrapped
    def D(self):
    	pass

    @methodnotwrapped
    def a(self):
    	pass

    @methodnotwrapped
    def t(self):
    	pass

    @methodnotwrapped
    def a(self):
    	pass

    @methodnotwrapped
    def _(self):
    	pass

    @methodnotwrapped
    def W(self):
    	pass

    @methodnotwrapped
    def i(self):
    	pass

    @methodnotwrapped
    def r(self):
    	pass

    @methodnotwrapped
    def e(self):
    	pass

    __swig_destroy__ = _BRepMeshData.delete_BRepMeshData_Wire

# Register BRepMeshData_Wire in _BRepMeshData:
_BRepMeshData.BRepMeshData_Wire_swigregister(BRepMeshData_Wire)








from enum import IntEnum
from typing import overload, NewType, Optional, Tuple

from OCC.Core.Standard import *
from OCC.Core.NCollection import *
from OCC.Core.Storage import *
from OCC.Core.TCollection import *
from OCC.Core.OSD import *
from OCC.Core.Resource import *


class utl:
	@staticmethod
	def AddToUserInfo(aData: Storage_Data, anInfo: TCollection_ExtendedString) -> None: ...
	@staticmethod
	def CString(anExtendedString: TCollection_ExtendedString) -> str: ...
	@staticmethod
	def Disk(aPath: OSD_Path) -> TCollection_ExtendedString: ...
	@staticmethod
	def ExtendedString(anAsciiString: TCollection_AsciiString) -> TCollection_ExtendedString: ...
	@overload
	@staticmethod
	def Extension(aPath: OSD_Path) -> TCollection_ExtendedString: ...
	@overload
	@staticmethod
	def Extension(aFileName: TCollection_ExtendedString) -> TCollection_ExtendedString: ...
	@staticmethod
	def FileIterator(aPath: OSD_Path, aMask: TCollection_ExtendedString) -> OSD_FileIterator: ...
	@staticmethod
	def Find(aResourceManager: Resource_Manager, aResourceName: TCollection_ExtendedString) -> bool: ...
	@staticmethod
	def GUID(anXString: TCollection_ExtendedString) -> Standard_GUID: ...
	@staticmethod
	def IntegerValue(anExtendedString: TCollection_ExtendedString) -> int: ...
	@staticmethod
	def IsReadOnly(aFileName: TCollection_ExtendedString) -> bool: ...
	@staticmethod
	def LocalHost() -> TCollection_ExtendedString: ...
	@staticmethod
	def Name(aPath: OSD_Path) -> TCollection_ExtendedString: ...
	@staticmethod
	def OpenFile(aFile: Storage_BaseDriver, aName: TCollection_ExtendedString, aMode: Storage_OpenMode) -> Storage_Error: ...
	@staticmethod
	def Path(aFileName: TCollection_ExtendedString) -> OSD_Path: ...
	@staticmethod
	def Trek(aPath: OSD_Path) -> TCollection_ExtendedString: ...
	@staticmethod
	def Value(aResourceManager: Resource_Manager, aResourceName: TCollection_ExtendedString) -> TCollection_ExtendedString: ...
	@staticmethod
	def xgetenv(aCString: str) -> TCollection_ExtendedString: ...

# harray1 classes
# harray2 classes
# hsequence classes

utl_AddToUserInfo = utl.AddToUserInfo
utl_CString = utl.CString
utl_Disk = utl.Disk
utl_ExtendedString = utl.ExtendedString
utl_Extension = utl.Extension
utl_Extension = utl.Extension
utl_FileIterator = utl.FileIterator
utl_Find = utl.Find
utl_GUID = utl.GUID
utl_IntegerValue = utl.IntegerValue
utl_IsReadOnly = utl.IsReadOnly
utl_LocalHost = utl.LocalHost
utl_Name = utl.Name
utl_OpenFile = utl.OpenFile
utl_Path = utl.Path
utl_Trek = utl.Trek
utl_Value = utl.Value
utl_xgetenv = utl.xgetenv



from enum import IntEnum
from typing import overload, NewType, Optional, Tuple

from OCC.Core.Standard import *
from OCC.Core.NCollection import *
from OCC.Core.Message import *
from OCC.Core.TDF import *
from OCC.Core.XmlObjMgt import *
from OCC.Core.TCollection import *


class xmlmdf:
	@staticmethod
	def AddDrivers(aDriverTable: XmlMDF_ADriverTable, theMessageDriver: Message_Messenger) -> None: ...
	@overload
	@staticmethod
	def FromTo(aSource: TDF_Data, aTarget: XmlObjMgt_Element, aReloc: XmlObjMgt_SRelocationTable, aDrivers: XmlMDF_ADriverTable, theRange: Optional[Message_ProgressRange] = Message_ProgressRange()) -> None: ...
	@overload
	@staticmethod
	def FromTo(aSource: XmlObjMgt_Element, aTarget: TDF_Data, aReloc: XmlObjMgt_RRelocationTable, aDrivers: XmlMDF_ADriverTable, theRange: Optional[Message_ProgressRange] = Message_ProgressRange()) -> bool: ...

class XmlMDF_ADriver(Standard_Transient):
	def MessageDriver(self) -> Message_Messenger: ...
	def Namespace(self) -> TCollection_AsciiString: ...
	def NewEmpty(self) -> TDF_Attribute: ...
	@overload
	def Paste(self, aSource: XmlObjMgt_Persistent, aTarget: TDF_Attribute, aRelocTable: XmlObjMgt_RRelocationTable) -> bool: ...
	@overload
	def Paste(self, aSource: TDF_Attribute, aTarget: XmlObjMgt_Persistent, aRelocTable: XmlObjMgt_SRelocationTable) -> None: ...
	def SourceType(self) -> Standard_Type: ...
	def TypeName(self) -> TCollection_AsciiString: ...
	def VersionNumber(self) -> int: ...

class XmlMDF_ADriverTable(Standard_Transient):
	def __init__(self) -> None: ...
	@overload
	def AddDerivedDriver(self, theInstance: TDF_Attribute) -> None: ...
	@overload
	def AddDerivedDriver(self, theDerivedType: str) -> Standard_Type: ...
	def AddDriver(self, anHDriver: XmlMDF_ADriver) -> None: ...
	def CreateDrvMap(self, theDriverMap: XmlMDF_MapOfDriver) -> None: ...
	def GetDriver(self, theType: Standard_Type, theDriver: XmlMDF_ADriver) -> bool: ...

class XmlMDF_DerivedDriver(XmlMDF_ADriver):
	def __init__(self, theDerivative: TDF_Attribute, theBaseDriver: XmlMDF_ADriver) -> None: ...
	def NewEmpty(self) -> TDF_Attribute: ...
	@overload
	def Paste(self, theSource: XmlObjMgt_Persistent, theTarget: TDF_Attribute, theRelocTable: XmlObjMgt_RRelocationTable) -> bool: ...
	@overload
	def Paste(self, theSource: TDF_Attribute, theTarget: XmlObjMgt_Persistent, theRelocTable: XmlObjMgt_SRelocationTable) -> None: ...
	def TypeName(self) -> TCollection_AsciiString: ...

class XmlMDF_ReferenceDriver(XmlMDF_ADriver):
	def __init__(self, theMessageDriver: Message_Messenger) -> None: ...
	def NewEmpty(self) -> TDF_Attribute: ...
	@overload
	def Paste(self, Source: XmlObjMgt_Persistent, Target: TDF_Attribute, RelocTable: XmlObjMgt_RRelocationTable) -> bool: ...
	@overload
	def Paste(self, Source: TDF_Attribute, Target: XmlObjMgt_Persistent, RelocTable: XmlObjMgt_SRelocationTable) -> None: ...

class XmlMDF_TagSourceDriver(XmlMDF_ADriver):
	def __init__(self, theMessageDriver: Message_Messenger) -> None: ...
	def NewEmpty(self) -> TDF_Attribute: ...
	@overload
	def Paste(self, Source: XmlObjMgt_Persistent, Target: TDF_Attribute, RelocTable: XmlObjMgt_RRelocationTable) -> bool: ...
	@overload
	def Paste(self, Source: TDF_Attribute, Target: XmlObjMgt_Persistent, RelocTable: XmlObjMgt_SRelocationTable) -> None: ...

# harray1 classes
# harray2 classes
# hsequence classes

xmlmdf_AddDrivers = xmlmdf.AddDrivers
xmlmdf_FromTo = xmlmdf.FromTo
xmlmdf_FromTo = xmlmdf.FromTo



# This file was automatically generated by SWIG (http://www.swig.org).
# Version 4.0.2
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.

"""
XmlMDF module, see official documentation at
https://www.opencascade.com/doc/occt-7.4.0/refman/html/package_xmlmdf.html
"""

from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

# Import the low-level C/C++ module
if __package__ or "." in __name__:
    from . import _XmlMDF
else:
    import _XmlMDF

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)


class SwigPyIterator(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined - class is abstract")
    __repr__ = _swig_repr
    __swig_destroy__ = _XmlMDF.delete_SwigPyIterator

    def value(self) -> "PyObject *":
        return _XmlMDF.SwigPyIterator_value(self)

    def incr(self, n: "size_t"=1) -> "swig::SwigPyIterator *":
        return _XmlMDF.SwigPyIterator_incr(self, n)

    def decr(self, n: "size_t"=1) -> "swig::SwigPyIterator *":
        return _XmlMDF.SwigPyIterator_decr(self, n)

    def distance(self, x: "SwigPyIterator") -> "ptrdiff_t":
        return _XmlMDF.SwigPyIterator_distance(self, x)

    def equal(self, x: "SwigPyIterator") -> "bool":
        return _XmlMDF.SwigPyIterator_equal(self, x)

    def copy(self) -> "swig::SwigPyIterator *":
        return _XmlMDF.SwigPyIterator_copy(self)

    def next(self) -> "PyObject *":
        return _XmlMDF.SwigPyIterator_next(self)

    def __next__(self) -> "PyObject *":
        return _XmlMDF.SwigPyIterator___next__(self)

    def previous(self) -> "PyObject *":
        return _XmlMDF.SwigPyIterator_previous(self)

    def advance(self, n: "ptrdiff_t") -> "swig::SwigPyIterator *":
        return _XmlMDF.SwigPyIterator_advance(self, n)

    def __eq__(self, x: "SwigPyIterator") -> "bool":
        return _XmlMDF.SwigPyIterator___eq__(self, x)

    def __ne__(self, x: "SwigPyIterator") -> "bool":
        return _XmlMDF.SwigPyIterator___ne__(self, x)

    def __iadd__(self, n: "ptrdiff_t") -> "swig::SwigPyIterator &":
        return _XmlMDF.SwigPyIterator___iadd__(self, n)

    def __isub__(self, n: "ptrdiff_t") -> "swig::SwigPyIterator &":
        return _XmlMDF.SwigPyIterator___isub__(self, n)

    def __add__(self, n: "ptrdiff_t") -> "swig::SwigPyIterator *":
        return _XmlMDF.SwigPyIterator___add__(self, n)

    def __sub__(self, *args) -> "ptrdiff_t":
        return _XmlMDF.SwigPyIterator___sub__(self, *args)
    def __iter__(self):
        return self

# Register SwigPyIterator in _XmlMDF:
_XmlMDF.SwigPyIterator_swigregister(SwigPyIterator)


def _dumps_object(klass):
    """ Overwrite default string output for any wrapped object.
    By default, __repr__ method returns something like:
    <OCC.Core.TopoDS.TopoDS_Shape; proxy of <Swig Object of type 'TopoDS_Shape *' at 0x02BB0758> >
    This is too much verbose.
    We prefer :
    <class 'gp_Pnt'>
    or
    <class 'TopoDS_Shape'>
    """
    klass_name = str(klass.__class__).split(".")[3].split("'")[0]
    repr_string = "<class '" + klass_name + "'"
# for TopoDS_Shape, we also look for the base type
    if klass_name == "TopoDS_Shape":
        if klass.IsNull():
            repr_string += ": Null>"
            return repr_string
        st = klass.ShapeType()
        types = {OCC.Core.TopAbs.TopAbs_VERTEX: "Vertex",
                 OCC.Core.TopAbs.TopAbs_SOLID: "Solid",
                 OCC.Core.TopAbs.TopAbs_EDGE: "Edge",
                 OCC.Core.TopAbs.TopAbs_FACE: "Face",
                 OCC.Core.TopAbs.TopAbs_SHELL: "Shell",
                 OCC.Core.TopAbs.TopAbs_WIRE: "Wire",
                 OCC.Core.TopAbs.TopAbs_COMPOUND: "Compound",
                 OCC.Core.TopAbs.TopAbs_COMPSOLID: "Compsolid"}
        repr_string += "; Type:%s" % types[st]        
    elif hasattr(klass, "IsNull"):
        if klass.IsNull():
            repr_string += "; Null"
    repr_string += ">"
    return repr_string


def process_exception(error: "Standard_Failure", method_name: "std::string", class_name: "std::string") -> "void":
    return _XmlMDF.process_exception(error, method_name, class_name)

from six import with_metaclass
import warnings
from OCC.Wrapper.wrapper_utils import Proxy, deprecated

import OCC.Core.Standard
import OCC.Core.NCollection
import OCC.Core.Message
import OCC.Core.TCollection
import OCC.Core.OSD
import OCC.Core.TColStd
import OCC.Core.TDF
import OCC.Core.XmlObjMgt
import OCC.Core.LDOM
import OCC.Core.gp
import OCC.Core.Storage

from enum import IntEnum
from OCC.Core.Exception import *




def Handle_XmlMDF_ADriver_Create() -> "opencascade::handle< XmlMDF_ADriver >":
    return _XmlMDF.Handle_XmlMDF_ADriver_Create()

def Handle_XmlMDF_ADriver_DownCast(t: "opencascade::handle< Standard_Transient > const &") -> "opencascade::handle< XmlMDF_ADriver >":
    return _XmlMDF.Handle_XmlMDF_ADriver_DownCast(t)

def Handle_XmlMDF_ADriver_IsNull(t: "opencascade::handle< XmlMDF_ADriver > const &") -> "bool":
    return _XmlMDF.Handle_XmlMDF_ADriver_IsNull(t)

def Handle_XmlMDF_ADriverTable_Create() -> "opencascade::handle< XmlMDF_ADriverTable >":
    return _XmlMDF.Handle_XmlMDF_ADriverTable_Create()

def Handle_XmlMDF_ADriverTable_DownCast(t: "opencascade::handle< Standard_Transient > const &") -> "opencascade::handle< XmlMDF_ADriverTable >":
    return _XmlMDF.Handle_XmlMDF_ADriverTable_DownCast(t)

def Handle_XmlMDF_ADriverTable_IsNull(t: "opencascade::handle< XmlMDF_ADriverTable > const &") -> "bool":
    return _XmlMDF.Handle_XmlMDF_ADriverTable_IsNull(t)

def Handle_XmlMDF_DerivedDriver_Create() -> "opencascade::handle< XmlMDF_DerivedDriver >":
    return _XmlMDF.Handle_XmlMDF_DerivedDriver_Create()

def Handle_XmlMDF_DerivedDriver_DownCast(t: "opencascade::handle< Standard_Transient > const &") -> "opencascade::handle< XmlMDF_DerivedDriver >":
    return _XmlMDF.Handle_XmlMDF_DerivedDriver_DownCast(t)

def Handle_XmlMDF_DerivedDriver_IsNull(t: "opencascade::handle< XmlMDF_DerivedDriver > const &") -> "bool":
    return _XmlMDF.Handle_XmlMDF_DerivedDriver_IsNull(t)

def Handle_XmlMDF_ReferenceDriver_Create() -> "opencascade::handle< XmlMDF_ReferenceDriver >":
    return _XmlMDF.Handle_XmlMDF_ReferenceDriver_Create()

def Handle_XmlMDF_ReferenceDriver_DownCast(t: "opencascade::handle< Standard_Transient > const &") -> "opencascade::handle< XmlMDF_ReferenceDriver >":
    return _XmlMDF.Handle_XmlMDF_ReferenceDriver_DownCast(t)

def Handle_XmlMDF_ReferenceDriver_IsNull(t: "opencascade::handle< XmlMDF_ReferenceDriver > const &") -> "bool":
    return _XmlMDF.Handle_XmlMDF_ReferenceDriver_IsNull(t)

def Handle_XmlMDF_TagSourceDriver_Create() -> "opencascade::handle< XmlMDF_TagSourceDriver >":
    return _XmlMDF.Handle_XmlMDF_TagSourceDriver_Create()

def Handle_XmlMDF_TagSourceDriver_DownCast(t: "opencascade::handle< Standard_Transient > const &") -> "opencascade::handle< XmlMDF_TagSourceDriver >":
    return _XmlMDF.Handle_XmlMDF_TagSourceDriver_DownCast(t)

def Handle_XmlMDF_TagSourceDriver_IsNull(t: "opencascade::handle< XmlMDF_TagSourceDriver > const &") -> "bool":
    return _XmlMDF.Handle_XmlMDF_TagSourceDriver_IsNull(t)
class XmlMDF_MapOfDriver(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def begin(self) -> "NCollection_DataMap< TCollection_AsciiString,opencascade::handle< XmlMDF_ADriver >,TCollection_AsciiString >::iterator":
        return _XmlMDF.XmlMDF_MapOfDriver_begin(self)

    def end(self) -> "NCollection_DataMap< TCollection_AsciiString,opencascade::handle< XmlMDF_ADriver >,TCollection_AsciiString >::iterator":
        return _XmlMDF.XmlMDF_MapOfDriver_end(self)

    def cbegin(self) -> "NCollection_DataMap< TCollection_AsciiString,opencascade::handle< XmlMDF_ADriver >,TCollection_AsciiString >::const_iterator":
        return _XmlMDF.XmlMDF_MapOfDriver_cbegin(self)

    def cend(self) -> "NCollection_DataMap< TCollection_AsciiString,opencascade::handle< XmlMDF_ADriver >,TCollection_AsciiString >::const_iterator":
        return _XmlMDF.XmlMDF_MapOfDriver_cend(self)

    def __init__(self, *args):
        _XmlMDF.XmlMDF_MapOfDriver_swiginit(self, _XmlMDF.new_XmlMDF_MapOfDriver(*args))

    def Exchange(self, theOther: "XmlMDF_MapOfDriver") -> "void":
        return _XmlMDF.XmlMDF_MapOfDriver_Exchange(self, theOther)

    def Assign(self, theOther: "XmlMDF_MapOfDriver") -> "NCollection_DataMap< TCollection_AsciiString,opencascade::handle< XmlMDF_ADriver >,TCollection_AsciiString > &":
        return _XmlMDF.XmlMDF_MapOfDriver_Assign(self, theOther)

    def Set(self, theOther: "XmlMDF_MapOfDriver") -> "NCollection_DataMap< TCollection_AsciiString,opencascade::handle< XmlMDF_ADriver >,TCollection_AsciiString > &":
        return _XmlMDF.XmlMDF_MapOfDriver_Set(self, theOther)

    def ReSize(self, N: "Standard_Integer const") -> "void":
        return _XmlMDF.XmlMDF_MapOfDriver_ReSize(self, N)

    def Bind(self, theKey: "TCollection_AsciiString", theItem: "opencascade::handle< XmlMDF_ADriver > const &") -> "Standard_Boolean":
        return _XmlMDF.XmlMDF_MapOfDriver_Bind(self, theKey, theItem)

    def Bound(self, theKey: "TCollection_AsciiString", theItem: "opencascade::handle< XmlMDF_ADriver > const &") -> "opencascade::handle< XmlMDF_ADriver > *":
        return _XmlMDF.XmlMDF_MapOfDriver_Bound(self, theKey, theItem)

    def IsBound(self, theKey: "TCollection_AsciiString") -> "Standard_Boolean":
        return _XmlMDF.XmlMDF_MapOfDriver_IsBound(self, theKey)

    def UnBind(self, theKey: "TCollection_AsciiString") -> "Standard_Boolean":
        return _XmlMDF.XmlMDF_MapOfDriver_UnBind(self, theKey)

    def Seek(self, theKey: "TCollection_AsciiString") -> "opencascade::handle< XmlMDF_ADriver > const *":
        return _XmlMDF.XmlMDF_MapOfDriver_Seek(self, theKey)

    def Find(self, *args) -> "Standard_Boolean":
        return _XmlMDF.XmlMDF_MapOfDriver_Find(self, *args)

    def ChangeSeek(self, theKey: "TCollection_AsciiString") -> "opencascade::handle< XmlMDF_ADriver > *":
        return _XmlMDF.XmlMDF_MapOfDriver_ChangeSeek(self, theKey)

    def ChangeFind(self, theKey: "TCollection_AsciiString") -> "opencascade::handle< XmlMDF_ADriver > &":
        return _XmlMDF.XmlMDF_MapOfDriver_ChangeFind(self, theKey)

    def __call__(self, *args) -> "opencascade::handle< XmlMDF_ADriver > &":
        return _XmlMDF.XmlMDF_MapOfDriver___call__(self, *args)

    def Clear(self, *args) -> "void":
        return _XmlMDF.XmlMDF_MapOfDriver_Clear(self, *args)
    __swig_destroy__ = _XmlMDF.delete_XmlMDF_MapOfDriver

    def Size(self) -> "Standard_Integer":
        return _XmlMDF.XmlMDF_MapOfDriver_Size(self)

# Register XmlMDF_MapOfDriver in _XmlMDF:
_XmlMDF.XmlMDF_MapOfDriver_swigregister(XmlMDF_MapOfDriver)

class XmlMDF_TypeADriverMap(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def begin(self) -> "NCollection_DataMap< opencascade::handle< Standard_Type >,opencascade::handle< XmlMDF_ADriver >,NCollection_DefaultHasher< opencascade::handle< Standard_Transient > > >::iterator":
        return _XmlMDF.XmlMDF_TypeADriverMap_begin(self)

    def end(self) -> "NCollection_DataMap< opencascade::handle< Standard_Type >,opencascade::handle< XmlMDF_ADriver >,NCollection_DefaultHasher< opencascade::handle< Standard_Transient > > >::iterator":
        return _XmlMDF.XmlMDF_TypeADriverMap_end(self)

    def cbegin(self) -> "NCollection_DataMap< opencascade::handle< Standard_Type >,opencascade::handle< XmlMDF_ADriver >,NCollection_DefaultHasher< opencascade::handle< Standard_Transient > > >::const_iterator":
        return _XmlMDF.XmlMDF_TypeADriverMap_cbegin(self)

    def cend(self) -> "NCollection_DataMap< opencascade::handle< Standard_Type >,opencascade::handle< XmlMDF_ADriver >,NCollection_DefaultHasher< opencascade::handle< Standard_Transient > > >::const_iterator":
        return _XmlMDF.XmlMDF_TypeADriverMap_cend(self)

    def __init__(self, *args):
        _XmlMDF.XmlMDF_TypeADriverMap_swiginit(self, _XmlMDF.new_XmlMDF_TypeADriverMap(*args))

    def Exchange(self, theOther: "XmlMDF_TypeADriverMap") -> "void":
        return _XmlMDF.XmlMDF_TypeADriverMap_Exchange(self, theOther)

    def Assign(self, theOther: "XmlMDF_TypeADriverMap") -> "NCollection_DataMap< opencascade::handle< Standard_Type >,opencascade::handle< XmlMDF_ADriver >,TColStd_MapTransientHasher > &":
        return _XmlMDF.XmlMDF_TypeADriverMap_Assign(self, theOther)

    def Set(self, theOther: "XmlMDF_TypeADriverMap") -> "NCollection_DataMap< opencascade::handle< Standard_Type >,opencascade::handle< XmlMDF_ADriver >,TColStd_MapTransientHasher > &":
        return _XmlMDF.XmlMDF_TypeADriverMap_Set(self, theOther)

    def ReSize(self, N: "Standard_Integer const") -> "void":
        return _XmlMDF.XmlMDF_TypeADriverMap_ReSize(self, N)

    def Bind(self, theKey: "opencascade::handle< Standard_Type > const &", theItem: "opencascade::handle< XmlMDF_ADriver > const &") -> "Standard_Boolean":
        return _XmlMDF.XmlMDF_TypeADriverMap_Bind(self, theKey, theItem)

    def Bound(self, theKey: "opencascade::handle< Standard_Type > const &", theItem: "opencascade::handle< XmlMDF_ADriver > const &") -> "opencascade::handle< XmlMDF_ADriver > *":
        return _XmlMDF.XmlMDF_TypeADriverMap_Bound(self, theKey, theItem)

    def IsBound(self, theKey: "opencascade::handle< Standard_Type > const &") -> "Standard_Boolean":
        return _XmlMDF.XmlMDF_TypeADriverMap_IsBound(self, theKey)

    def UnBind(self, theKey: "opencascade::handle< Standard_Type > const &") -> "Standard_Boolean":
        return _XmlMDF.XmlMDF_TypeADriverMap_UnBind(self, theKey)

    def Seek(self, theKey: "opencascade::handle< Standard_Type > const &") -> "opencascade::handle< XmlMDF_ADriver > const *":
        return _XmlMDF.XmlMDF_TypeADriverMap_Seek(self, theKey)

    def Find(self, *args) -> "Standard_Boolean":
        return _XmlMDF.XmlMDF_TypeADriverMap_Find(self, *args)

    def ChangeSeek(self, theKey: "opencascade::handle< Standard_Type > const &") -> "opencascade::handle< XmlMDF_ADriver > *":
        return _XmlMDF.XmlMDF_TypeADriverMap_ChangeSeek(self, theKey)

    def ChangeFind(self, theKey: "opencascade::handle< Standard_Type > const &") -> "opencascade::handle< XmlMDF_ADriver > &":
        return _XmlMDF.XmlMDF_TypeADriverMap_ChangeFind(self, theKey)

    def __call__(self, *args) -> "opencascade::handle< XmlMDF_ADriver > &":
        return _XmlMDF.XmlMDF_TypeADriverMap___call__(self, *args)

    def Clear(self, *args) -> "void":
        return _XmlMDF.XmlMDF_TypeADriverMap_Clear(self, *args)
    __swig_destroy__ = _XmlMDF.delete_XmlMDF_TypeADriverMap

    def Size(self) -> "Standard_Integer":
        return _XmlMDF.XmlMDF_TypeADriverMap_Size(self)

# Register XmlMDF_TypeADriverMap in _XmlMDF:
_XmlMDF.XmlMDF_TypeADriverMap_swigregister(XmlMDF_TypeADriverMap)

class xmlmdf(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    @staticmethod
    def AddDrivers(*args) -> "void":
        r"""
        Adds the attribute storage drivers to <adriverseq>.

        Parameters
        ----------
        aDriverTable: XmlMDF_ADriverTable
        theMessageDriver: Message_Messenger

        Returns
        -------
        None

        """
        return _XmlMDF.xmlmdf_AddDrivers(*args)

    @staticmethod
    def FromTo(*args) -> "Standard_Boolean":
        r"""
        Translates a transient <asource> into a persistent <atarget>.

        Parameters
        ----------
        aSource: TDF_Data
        aTarget: XmlObjMgt_Element
        aReloc: XmlObjMgt_SRelocationTable
        aDrivers: XmlMDF_ADriverTable
        theRange: Message_ProgressRange,optional
        	default value is Message_ProgressRange()

        Returns
        -------
        None
        Translates a persistent <asource> into a transient <atarget>. returns true if completed successfully (false on error).

        Parameters
        ----------
        aSource: XmlObjMgt_Element
        aTarget: TDF_Data
        aReloc: XmlObjMgt_RRelocationTable
        aDrivers: XmlMDF_ADriverTable
        theRange: Message_ProgressRange,optional
        	default value is Message_ProgressRange()

        Returns
        -------
        bool

        """
        return _XmlMDF.xmlmdf_FromTo(*args)

    __repr__ = _dumps_object


    def __init__(self):
        _XmlMDF.xmlmdf_swiginit(self, _XmlMDF.new_xmlmdf())
    __swig_destroy__ = _XmlMDF.delete_xmlmdf

# Register xmlmdf in _XmlMDF:
_XmlMDF.xmlmdf_swigregister(xmlmdf)

def xmlmdf_AddDrivers(*args) -> "void":
    r"""
    Adds the attribute storage drivers to <adriverseq>.

    Parameters
    ----------
    aDriverTable: XmlMDF_ADriverTable
    theMessageDriver: Message_Messenger

    Returns
    -------
    None

    """
    return _XmlMDF.xmlmdf_AddDrivers(*args)

def xmlmdf_FromTo(*args) -> "Standard_Boolean":
    r"""
    Translates a transient <asource> into a persistent <atarget>.

    Parameters
    ----------
    aSource: TDF_Data
    aTarget: XmlObjMgt_Element
    aReloc: XmlObjMgt_SRelocationTable
    aDrivers: XmlMDF_ADriverTable
    theRange: Message_ProgressRange,optional
    	default value is Message_ProgressRange()

    Returns
    -------
    None
    Translates a persistent <asource> into a transient <atarget>. returns true if completed successfully (false on error).

    Parameters
    ----------
    aSource: XmlObjMgt_Element
    aTarget: TDF_Data
    aReloc: XmlObjMgt_RRelocationTable
    aDrivers: XmlMDF_ADriverTable
    theRange: Message_ProgressRange,optional
    	default value is Message_ProgressRange()

    Returns
    -------
    bool

    """
    return _XmlMDF.xmlmdf_FromTo(*args)

class XmlMDF_ADriver(OCC.Core.Standard.Standard_Transient):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined")
    __repr__ = _swig_repr

    def MessageDriver(self, *args) -> "opencascade::handle< Message_Messenger > const &":
        r"""
        Returns the current message driver of this driver.

        Returns
        -------
        opencascade::handle<Message_Messenger>

        """
        return _XmlMDF.XmlMDF_ADriver_MessageDriver(self, *args)

    def Namespace(self, *args) -> "TCollection_AsciiString const &":
        r"""
        Returns the namespace string.

        Returns
        -------
        TCollection_AsciiString

        """
        return _XmlMDF.XmlMDF_ADriver_Namespace(self, *args)

    def NewEmpty(self, *args) -> "opencascade::handle< TDF_Attribute >":
        r"""
        Creates a new attribute from tdf.

        Returns
        -------
        opencascade::handle<TDF_Attribute>

        """
        return _XmlMDF.XmlMDF_ADriver_NewEmpty(self, *args)

    def Paste(self, *args) -> "void":
        r"""
        Translate the contents of <asource> and put it into <atarget>, using the relocation table <areloctable> to keep the sharings.

        Parameters
        ----------
        aSource: XmlObjMgt_Persistent
        aTarget: TDF_Attribute
        aRelocTable: XmlObjMgt_RRelocationTable

        Returns
        -------
        bool
        Translate the contents of <asource> and put it into <atarget>, using the relocation table <areloctable> to keep the sharings.

        Parameters
        ----------
        aSource: TDF_Attribute
        aTarget: XmlObjMgt_Persistent
        aRelocTable: XmlObjMgt_SRelocationTable

        Returns
        -------
        None

        """
        return _XmlMDF.XmlMDF_ADriver_Paste(self, *args)

    def SourceType(self, *args) -> "opencascade::handle< Standard_Type >":
        r"""
        Returns the type of source object, inheriting from attribute from tdf.

        Returns
        -------
        opencascade::handle<Standard_Type>

        """
        return _XmlMDF.XmlMDF_ADriver_SourceType(self, *args)

    def TypeName(self, *args) -> "TCollection_AsciiString const &":
        r"""
        Returns the full xml tag name (including ns prefix).

        Returns
        -------
        TCollection_AsciiString

        """
        return _XmlMDF.XmlMDF_ADriver_TypeName(self, *args)

    def VersionNumber(self, *args) -> "Standard_Integer":
        r"""
        Returns the version number from which the driver is available.

        Returns
        -------
        int

        """
        return _XmlMDF.XmlMDF_ADriver_VersionNumber(self, *args)


    @staticmethod
    def DownCast(t):
      return Handle_XmlMDF_ADriver_DownCast(t)


    __repr__ = _dumps_object

    __swig_destroy__ = _XmlMDF.delete_XmlMDF_ADriver

# Register XmlMDF_ADriver in _XmlMDF:
_XmlMDF.XmlMDF_ADriver_swigregister(XmlMDF_ADriver)

class XmlMDF_ADriverTable(OCC.Core.Standard.Standard_Transient):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        Creates a mutable adrivertable from xmlmdf.

        Returns
        -------
        None

        """
        _XmlMDF.XmlMDF_ADriverTable_swiginit(self, _XmlMDF.new_XmlMDF_ADriverTable(*args))

    def AddDerivedDriver(self, *args) -> "opencascade::handle< Standard_Type > const &":
        r"""
        Adds a translation driver for the derived attribute. the base driver must be already added. @param theinstance is newly created attribute, detached from any label.

        Parameters
        ----------
        theInstance: TDF_Attribute

        Returns
        -------
        None
        Adds a translation driver for the derived attribute. the base driver must be already added. @param thederivedtype is registered attribute type using implement_derived_attribute macro.

        Parameters
        ----------
        theDerivedType: char *

        Returns
        -------
        opencascade::handle<Standard_Type>

        """
        return _XmlMDF.XmlMDF_ADriverTable_AddDerivedDriver(self, *args)

    def AddDriver(self, *args) -> "void":
        r"""
        Sets a translation driver: <adriver>.

        Parameters
        ----------
        anHDriver: XmlMDF_ADriver

        Returns
        -------
        None

        """
        return _XmlMDF.XmlMDF_ADriverTable_AddDriver(self, *args)

    def CreateDrvMap(self, *args) -> "void":
        r"""
        Fills the map by all registered drivers.

        Parameters
        ----------
        theDriverMap: XmlMDF_MapOfDriver

        Returns
        -------
        None

        """
        return _XmlMDF.XmlMDF_ADriverTable_CreateDrvMap(self, *args)

    def GetDriver(self, *args) -> "Standard_Boolean":
        r"""
        Gets a driver <adriver> according to <atype> //! returns true if a driver is found; false otherwise.

        Parameters
        ----------
        theType: Standard_Type
        theDriver: XmlMDF_ADriver

        Returns
        -------
        bool

        """
        return _XmlMDF.XmlMDF_ADriverTable_GetDriver(self, *args)


    @staticmethod
    def DownCast(t):
      return Handle_XmlMDF_ADriverTable_DownCast(t)


    __repr__ = _dumps_object

    __swig_destroy__ = _XmlMDF.delete_XmlMDF_ADriverTable

# Register XmlMDF_ADriverTable in _XmlMDF:
_XmlMDF.XmlMDF_ADriverTable_swigregister(XmlMDF_ADriverTable)

class XmlMDF_DerivedDriver(XmlMDF_ADriver):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        Creates a derivative persistence driver for thederivative attribute by reusage of thebasedriver @param thederivative an instance of the attribute, just created, detached from any label @param thebasedriver a driver of the base attribute, called by paste methods.

        Parameters
        ----------
        theDerivative: TDF_Attribute
        theBaseDriver: XmlMDF_ADriver

        Returns
        -------
        None

        """
        _XmlMDF.XmlMDF_DerivedDriver_swiginit(self, _XmlMDF.new_XmlMDF_DerivedDriver(*args))

    def Paste(self, *args) -> "void":
        r"""
        Reuses the base driver to read the base fields.

        Parameters
        ----------
        theSource: XmlObjMgt_Persistent
        theTarget: TDF_Attribute
        theRelocTable: XmlObjMgt_RRelocationTable

        Returns
        -------
        bool
        Reuses the base driver to store the base fields.

        Parameters
        ----------
        theSource: TDF_Attribute
        theTarget: XmlObjMgt_Persistent
        theRelocTable: XmlObjMgt_SRelocationTable

        Returns
        -------
        None

        """
        return _XmlMDF.XmlMDF_DerivedDriver_Paste(self, *args)

    def TypeName(self, *args) -> "TCollection_AsciiString const &":
        r"""
        Returns the full xml tag name (including ns prefix).

        Returns
        -------
        TCollection_AsciiString

        """
        return _XmlMDF.XmlMDF_DerivedDriver_TypeName(self, *args)


    @staticmethod
    def DownCast(t):
      return Handle_XmlMDF_DerivedDriver_DownCast(t)


    __repr__ = _dumps_object

    __swig_destroy__ = _XmlMDF.delete_XmlMDF_DerivedDriver

# Register XmlMDF_DerivedDriver in _XmlMDF:
_XmlMDF.XmlMDF_DerivedDriver_swigregister(XmlMDF_DerivedDriver)

class XmlMDF_ReferenceDriver(XmlMDF_ADriver):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        No available documentation.

        Parameters
        ----------
        theMessageDriver: Message_Messenger

        Returns
        -------
        None

        """
        _XmlMDF.XmlMDF_ReferenceDriver_swiginit(self, _XmlMDF.new_XmlMDF_ReferenceDriver(*args))

    def Paste(self, *args) -> "void":
        r"""
        No available documentation.

        Parameters
        ----------
        Source: XmlObjMgt_Persistent
        Target: TDF_Attribute
        RelocTable: XmlObjMgt_RRelocationTable

        Returns
        -------
        bool
        No available documentation.

        Parameters
        ----------
        Source: TDF_Attribute
        Target: XmlObjMgt_Persistent
        RelocTable: XmlObjMgt_SRelocationTable

        Returns
        -------
        None

        """
        return _XmlMDF.XmlMDF_ReferenceDriver_Paste(self, *args)


    @staticmethod
    def DownCast(t):
      return Handle_XmlMDF_ReferenceDriver_DownCast(t)


    __repr__ = _dumps_object

    __swig_destroy__ = _XmlMDF.delete_XmlMDF_ReferenceDriver

# Register XmlMDF_ReferenceDriver in _XmlMDF:
_XmlMDF.XmlMDF_ReferenceDriver_swigregister(XmlMDF_ReferenceDriver)

class XmlMDF_TagSourceDriver(XmlMDF_ADriver):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        No available documentation.

        Parameters
        ----------
        theMessageDriver: Message_Messenger

        Returns
        -------
        None

        """
        _XmlMDF.XmlMDF_TagSourceDriver_swiginit(self, _XmlMDF.new_XmlMDF_TagSourceDriver(*args))

    def Paste(self, *args) -> "void":
        r"""
        No available documentation.

        Parameters
        ----------
        Source: XmlObjMgt_Persistent
        Target: TDF_Attribute
        RelocTable: XmlObjMgt_RRelocationTable

        Returns
        -------
        bool
        No available documentation.

        Parameters
        ----------
        Source: TDF_Attribute
        Target: XmlObjMgt_Persistent
        RelocTable: XmlObjMgt_SRelocationTable

        Returns
        -------
        None

        """
        return _XmlMDF.XmlMDF_TagSourceDriver_Paste(self, *args)


    @staticmethod
    def DownCast(t):
      return Handle_XmlMDF_TagSourceDriver_DownCast(t)


    __repr__ = _dumps_object

    __swig_destroy__ = _XmlMDF.delete_XmlMDF_TagSourceDriver

# Register XmlMDF_TagSourceDriver in _XmlMDF:
_XmlMDF.XmlMDF_TagSourceDriver_swigregister(XmlMDF_TagSourceDriver)








from enum import IntEnum
from typing import overload, NewType, Optional, Tuple

from OCC.Core.Standard import *
from OCC.Core.NCollection import *
from OCC.Core.Graphic3d import *
from OCC.Core.gp import *
from OCC.Core.Select3D import *
from OCC.Core.PrsMgr import *
from OCC.Core.Aspect import *
from OCC.Core.Prs3d import *
from OCC.Core.TopLoc import *
from OCC.Core.V3d import *
from OCC.Core.TopAbs import *
from OCC.Core.BVH import *
from OCC.Core.SelectBasics import *
from OCC.Core.TColgp import *
from OCC.Core.Bnd import *

SelectBasics_EntityOwner = NewType('SelectBasics_EntityOwner', SelectMgr_EntityOwner)
#the following typedef cannot be wrapped as is
SelectMgr_IndexedDataMapOfOwnerCriterion = NewType('SelectMgr_IndexedDataMapOfOwnerCriterion', Any)
#the following typedef cannot be wrapped as is
SelectMgr_IndexedMapOfHSensitive = NewType('SelectMgr_IndexedMapOfHSensitive', Any)
#the following typedef cannot be wrapped as is
SelectMgr_IndexedMapOfOwner = NewType('SelectMgr_IndexedMapOfOwner', Any)
#the following typedef cannot be wrapped as is
SelectMgr_Mat4 = NewType('SelectMgr_Mat4', Any)
#the following typedef cannot be wrapped as is
SelectMgr_Vec3 = NewType('SelectMgr_Vec3', Any)
#the following typedef cannot be wrapped as is
SelectMgr_Vec4 = NewType('SelectMgr_Vec4', Any)

class SelectMgr_ListOfFilter:
    def __init__(self) -> None: ...
    def __len__(self) -> int: ...
    def Size(self) -> int: ...
    def Clear(self) -> None: ...
    def First(self) -> False: ...
    def Last(self) -> False: ...
    def Append(self, theItem: False) -> False: ...
    def Prepend(self, theItem: False) -> False: ...
    def RemoveFirst(self) -> None: ...
    def Reverse(self) -> None: ...
    def Value(self, theIndex: int) -> False: ...
    def SetValue(self, theIndex: int, theValue: False) -> None: ...

class SelectMgr_SequenceOfFilter:
    def __init__(self) -> None: ...
    def __len__(self) -> int: ...
    def Size(self) -> int: ...
    def Clear(self) -> None: ...
    def First(self) -> False: ...
    def Last(self) -> False: ...
    def Length(self) -> int: ...
    def Append(self, theItem: False) -> False: ...
    def Prepend(self, theItem: False) -> False: ...
    def RemoveFirst(self) -> None: ...
    def Reverse(self) -> None: ...
    def Value(self, theIndex: int) -> False: ...
    def SetValue(self, theIndex: int, theValue: False) -> None: ...

class SelectMgr_SequenceOfOwner:
    def __init__(self) -> None: ...
    def __len__(self) -> int: ...
    def Size(self) -> int: ...
    def Clear(self) -> None: ...
    def First(self) -> False: ...
    def Last(self) -> False: ...
    def Length(self) -> int: ...
    def Append(self, theItem: False) -> False: ...
    def Prepend(self, theItem: False) -> False: ...
    def RemoveFirst(self) -> None: ...
    def Reverse(self) -> None: ...
    def Value(self, theIndex: int) -> False: ...
    def SetValue(self, theIndex: int, theValue: False) -> None: ...

class SelectMgr_SequenceOfSelection:
    def __init__(self) -> None: ...
    def __len__(self) -> int: ...
    def Size(self) -> int: ...
    def Clear(self) -> None: ...
    def First(self) -> False: ...
    def Last(self) -> False: ...
    def Length(self) -> int: ...
    def Append(self, theItem: False) -> False: ...
    def Prepend(self, theItem: False) -> False: ...
    def RemoveFirst(self) -> None: ...
    def Reverse(self) -> None: ...
    def Value(self, theIndex: int) -> False: ...
    def SetValue(self, theIndex: int, theValue: False) -> None: ...

class SelectMgr_FilterType(IntEnum):
	SelectMgr_FilterType_AND: int = ...
	SelectMgr_FilterType_OR: int = ...
SelectMgr_FilterType_AND = SelectMgr_FilterType.SelectMgr_FilterType_AND
SelectMgr_FilterType_OR = SelectMgr_FilterType.SelectMgr_FilterType_OR

class SelectMgr_TypeOfDepthTolerance(IntEnum):
	SelectMgr_TypeOfDepthTolerance_Uniform: int = ...
	SelectMgr_TypeOfDepthTolerance_UniformPixels: int = ...
	SelectMgr_TypeOfDepthTolerance_SensitivityFactor: int = ...
SelectMgr_TypeOfDepthTolerance_Uniform = SelectMgr_TypeOfDepthTolerance.SelectMgr_TypeOfDepthTolerance_Uniform
SelectMgr_TypeOfDepthTolerance_UniformPixels = SelectMgr_TypeOfDepthTolerance.SelectMgr_TypeOfDepthTolerance_UniformPixels
SelectMgr_TypeOfDepthTolerance_SensitivityFactor = SelectMgr_TypeOfDepthTolerance.SelectMgr_TypeOfDepthTolerance_SensitivityFactor

class SelectMgr_TypeOfUpdate(IntEnum):
	SelectMgr_TOU_Full: int = ...
	SelectMgr_TOU_Partial: int = ...
	SelectMgr_TOU_None: int = ...
SelectMgr_TOU_Full = SelectMgr_TypeOfUpdate.SelectMgr_TOU_Full
SelectMgr_TOU_Partial = SelectMgr_TypeOfUpdate.SelectMgr_TOU_Partial
SelectMgr_TOU_None = SelectMgr_TypeOfUpdate.SelectMgr_TOU_None

class SelectMgr_TypeOfBVHUpdate(IntEnum):
	SelectMgr_TBU_Add: int = ...
	SelectMgr_TBU_Remove: int = ...
	SelectMgr_TBU_Renew: int = ...
	SelectMgr_TBU_Invalidate: int = ...
	SelectMgr_TBU_None: int = ...
SelectMgr_TBU_Add = SelectMgr_TypeOfBVHUpdate.SelectMgr_TBU_Add
SelectMgr_TBU_Remove = SelectMgr_TypeOfBVHUpdate.SelectMgr_TBU_Remove
SelectMgr_TBU_Renew = SelectMgr_TypeOfBVHUpdate.SelectMgr_TBU_Renew
SelectMgr_TBU_Invalidate = SelectMgr_TypeOfBVHUpdate.SelectMgr_TBU_Invalidate
SelectMgr_TBU_None = SelectMgr_TypeOfBVHUpdate.SelectMgr_TBU_None

class SelectMgr_StateOfSelection(IntEnum):
	SelectMgr_SOS_Any: int = ...
	SelectMgr_SOS_Unknown: int = ...
	SelectMgr_SOS_Deactivated: int = ...
	SelectMgr_SOS_Activated: int = ...
SelectMgr_SOS_Any = SelectMgr_StateOfSelection.SelectMgr_SOS_Any
SelectMgr_SOS_Unknown = SelectMgr_StateOfSelection.SelectMgr_SOS_Unknown
SelectMgr_SOS_Deactivated = SelectMgr_StateOfSelection.SelectMgr_SOS_Deactivated
SelectMgr_SOS_Activated = SelectMgr_StateOfSelection.SelectMgr_SOS_Activated

class SelectMgr_PickingStrategy(IntEnum):
	SelectMgr_PickingStrategy_FirstAcceptable: int = ...
	SelectMgr_PickingStrategy_OnlyTopmost: int = ...
SelectMgr_PickingStrategy_FirstAcceptable = SelectMgr_PickingStrategy.SelectMgr_PickingStrategy_FirstAcceptable
SelectMgr_PickingStrategy_OnlyTopmost = SelectMgr_PickingStrategy.SelectMgr_PickingStrategy_OnlyTopmost

class selectmgr:
	@staticmethod
	def ComputeSensitivePrs(theStructure: Graphic3d_Structure, theSel: SelectMgr_Selection, theLoc: gp_Trsf, theTrsfPers: Graphic3d_TransformPers) -> None: ...

class SelectMgr_BVHThreadPool(Standard_Transient):
	def __init__(self, theNbThreads: int) -> None: ...
	def AddEntity(self, theEntity: Select3D_SensitiveEntity) -> None: ...
	def StopThreads(self) -> None: ...
	def Threads(self) -> False: ...
	def WaitThreads(self) -> None: ...

class SelectMgr_EntityOwner(Standard_Transient):
	@overload
	def __init__(self, aPriority: Optional[int] = 0) -> None: ...
	@overload
	def __init__(self, aSO: SelectMgr_SelectableObject, aPriority: Optional[int] = 0) -> None: ...
	@overload
	def __init__(self, theOwner: SelectMgr_EntityOwner, aPriority: Optional[int] = 0) -> None: ...
	def Clear(self, thePrsMgr: PrsMgr_PresentationManager, theMode: Optional[int] = 0) -> None: ...
	def ComesFromDecomposition(self) -> bool: ...
	def HasLocation(self) -> bool: ...
	def HasSelectable(self) -> bool: ...
	def HilightWithColor(self, thePrsMgr: PrsMgr_PresentationManager, theStyle: Prs3d_Drawer, theMode: Optional[int] = 0) -> None: ...
	def IsAutoHilight(self) -> bool: ...
	def IsForcedHilight(self) -> bool: ...
	def IsHilighted(self, thePrsMgr: PrsMgr_PresentationManager, theMode: Optional[int] = 0) -> bool: ...
	def IsSameSelectable(self, theOther: SelectMgr_SelectableObject) -> bool: ...
	def IsSelected(self) -> bool: ...
	def Location(self) -> TopLoc_Location: ...
	def Priority(self) -> int: ...
	def Selectable(self) -> SelectMgr_SelectableObject: ...
	@overload
	def Set(self, theSelObj: SelectMgr_SelectableObject) -> None: ...
	@overload
	def Set(self, thePriority: int) -> None: ...
	def SetComesFromDecomposition(self, theIsFromDecomposition: bool) -> None: ...
	def SetLocation(self, theLocation: TopLoc_Location) -> None: ...
	def SetPriority(self, thePriority: int) -> None: ...
	def SetSelectable(self, theSelObj: SelectMgr_SelectableObject) -> None: ...
	def SetSelected(self, theIsSelected: bool) -> None: ...
	@overload
	def State(self) -> int: ...
	@overload
	def State(self, theStatus: int) -> None: ...
	def Unhilight(self, thePrsMgr: PrsMgr_PresentationManager, theMode: Optional[int] = 0) -> None: ...
	def UpdateHighlightTrsf(self, theViewer: V3d_Viewer, theManager: PrsMgr_PresentationManager, theDispMode: int) -> None: ...

class SelectMgr_Filter(Standard_Transient):
	def ActsOn(self, aStandardMode: TopAbs_ShapeEnum) -> bool: ...
	def IsOk(self, anObj: SelectMgr_EntityOwner) -> bool: ...

class SelectMgr_SelectableObject(PrsMgr_PresentableObject):
	def AcceptShapeDecomposition(self) -> bool: ...
	def AddSelection(self, aSelection: SelectMgr_Selection, aMode: int) -> None: ...
	def ClearDynamicHighlight(self, theMgr: PrsMgr_PresentationManager) -> None: ...
	def ClearSelected(self) -> None: ...
	def ClearSelections(self, update: Optional[bool] = False) -> None: ...
	def ComputeSelection(self, theSelection: SelectMgr_Selection, theMode: int) -> None: ...
	def ErasePresentations(self, theToRemove: bool) -> None: ...
	def GetAssemblyOwner(self) -> SelectMgr_EntityOwner: ...
	def GetHilightPresentation(self, thePrsMgr: PrsMgr_PresentationManager) -> Prs3d_Presentation: ...
	def GetSelectPresentation(self, thePrsMgr: PrsMgr_PresentationManager) -> Prs3d_Presentation: ...
	def GlobalSelOwner(self) -> SelectMgr_EntityOwner: ...
	def GlobalSelectionMode(self) -> int: ...
	def HasSelection(self, theMode: int) -> bool: ...
	def HilightOwnerWithColor(self, thePM: PrsMgr_PresentationManager, theStyle: Prs3d_Drawer, theOwner: SelectMgr_EntityOwner) -> None: ...
	def HilightSelected(self, thePrsMgr: PrsMgr_PresentationManager, theSeq: SelectMgr_SequenceOfOwner) -> None: ...
	def IsAutoHilight(self) -> bool: ...
	@overload
	def RecomputePrimitives(self) -> None: ...
	@overload
	def RecomputePrimitives(self, theMode: int) -> None: ...
	def ResetTransformation(self) -> None: ...
	def Selection(self, theMode: int) -> SelectMgr_Selection: ...
	def Selections(self) -> SelectMgr_SequenceOfSelection: ...
	def SetAssemblyOwner(self, theOwner: SelectMgr_EntityOwner, theMode: Optional[int] = -1) -> None: ...
	def SetAutoHilight(self, theAutoHilight: bool) -> None: ...
	def UpdateSelection(self, theMode: Optional[int] = -1) -> None: ...
	def UpdateTransformation(self) -> None: ...
	def UpdateTransformations(self, aSelection: SelectMgr_Selection) -> None: ...

class SelectMgr_SelectableObjectSet:
	def __init__(self) -> None: ...
	def Append(self, theObject: SelectMgr_SelectableObject) -> bool: ...
	def ChangeSubset(self, theObject: SelectMgr_SelectableObject) -> None: ...
	def Contains(self, theObject: SelectMgr_SelectableObject) -> bool: ...
	@overload
	def IsEmpty(self) -> bool: ...
	def MarkDirty(self) -> None: ...
	def Remove(self, theObject: SelectMgr_SelectableObject) -> bool: ...
	def UpdateBVH(self, theCamera: Graphic3d_Camera, theProjectionMat: Graphic3d_Mat4d, theWorldViewMat: Graphic3d_Mat4d, theViewState: Graphic3d_WorldViewProjState, theViewportWidth: int, theViewportHeight: int) -> None: ...

class SelectMgr_SelectingVolumeManager(SelectBasics_SelectingVolumeManager):
	def __init__(self, theToAllocateFrustums: Optional[bool] = True) -> None: ...
	def AllowOverlapDetection(self, theIsToAllow: bool) -> None: ...
	@overload
	def BuildSelectingVolume(self, thePoint: gp_Pnt2d) -> None: ...
	@overload
	def BuildSelectingVolume(self, theMinPt: gp_Pnt2d, theMaxPt: gp_Pnt2d) -> None: ...
	@overload
	def BuildSelectingVolume(self, thePoints: TColgp_Array1OfPnt2d) -> None: ...
	def Camera(self) -> Graphic3d_Camera: ...
	def DetectedPoint(self, theDepth: float) -> gp_Pnt: ...
	def DistToGeometryCenter(self, theCOG: gp_Pnt) -> float: ...
	def GetActiveSelectionType(self) -> int: ...
	def GetFarPickedPnt(self) -> gp_Pnt: ...
	def GetMousePosition(self) -> gp_Pnt2d: ...
	def GetNearPickedPnt(self) -> gp_Pnt: ...
	def GetVertices(self) -> gp_Pnt: ...
	def IsOverlapAllowed(self) -> bool: ...
	def ObjectClipping(self) -> Graphic3d_SequenceOfHClipPlane: ...
	@overload
	def Overlaps(self, theBoxMin: SelectMgr_Vec3, theBoxMax: SelectMgr_Vec3, thePickResult: SelectBasics_PickResult) -> bool: ...
	@overload
	def Overlaps(self, theBoxMin: SelectMgr_Vec3, theBoxMax: SelectMgr_Vec3, theInside: Optional[bool] = None) -> bool: ...
	@overload
	def Overlaps(self, thePnt: gp_Pnt, thePickResult: SelectBasics_PickResult) -> bool: ...
	@overload
	def Overlaps(self, thePnt: gp_Pnt) -> bool: ...
	@overload
	def Overlaps(self, theArrayOfPts: TColgp_HArray1OfPnt, theSensType: int, thePickResult: SelectBasics_PickResult) -> bool: ...
	@overload
	def Overlaps(self, theArrayOfPts: TColgp_Array1OfPnt, theSensType: int, thePickResult: SelectBasics_PickResult) -> bool: ...
	@overload
	def Overlaps(self, thePnt1: gp_Pnt, thePnt2: gp_Pnt, thePickResult: SelectBasics_PickResult) -> bool: ...
	@overload
	def Overlaps(self, thePnt1: gp_Pnt, thePnt2: gp_Pnt, thePnt3: gp_Pnt, theSensType: int, thePickResult: SelectBasics_PickResult) -> bool: ...
	def ProjectionMatrix(self) -> Graphic3d_Mat4d: ...
	def ScaleAndTransform(self, theScaleFactor: int, theTrsf: gp_GTrsf, theBuilder: Optional[SelectMgr_FrustumBuilder] = None) -> SelectMgr_SelectingVolumeManager: ...
	@overload
	def SetCamera(self, theCamera: Graphic3d_Camera) -> None: ...
	@overload
	def SetCamera(self, theProjection: Graphic3d_Mat4d, theWorldView: Graphic3d_Mat4d, theIsOrthographic: bool, theWVPState: Optional[Graphic3d_WorldViewProjState] = Graphic3d_WorldViewProjState()) -> None: ...
	def SetPixelTolerance(self, theTolerance: int) -> None: ...
	def SetViewClipRanges(self, theRange: SelectMgr_ViewClipRange) -> None: ...
	@overload
	def SetViewClipping(self, theViewPlanes: Graphic3d_SequenceOfHClipPlane, theObjPlanes: Graphic3d_SequenceOfHClipPlane, theWorldSelMgr: SelectMgr_SelectingVolumeManager) -> None: ...
	@overload
	def SetViewClipping(self, theOther: SelectMgr_SelectingVolumeManager) -> None: ...
	def SetViewport(self, theX: float, theY: float, theWidth: float, theHeight: float) -> None: ...
	def SetWindowSize(self, theWidth: int, theHeight: int) -> None: ...
	def ViewClipRanges(self) -> SelectMgr_ViewClipRange: ...
	def ViewClipping(self) -> Graphic3d_SequenceOfHClipPlane: ...
	def WindowSize(self) -> Tuple[int, int]: ...
	def WorldViewMatrix(self) -> Graphic3d_Mat4d: ...
	def WorldViewProjState(self) -> Graphic3d_WorldViewProjState: ...

class SelectMgr_Selection(Standard_Transient):
	def __init__(self, theModeIdx: Optional[int] = 0) -> None: ...
	def Add(self, theSensitive: Select3D_SensitiveEntity) -> None: ...
	def BVHUpdateStatus(self) -> SelectMgr_TypeOfBVHUpdate: ...
	def ChangeEntities(self) -> False: ...
	def Clear(self) -> None: ...
	def Destroy(self) -> None: ...
	def Entities(self) -> False: ...
	def GetSelectionState(self) -> SelectMgr_StateOfSelection: ...
	def IsEmpty(self) -> bool: ...
	def Mode(self) -> int: ...
	def Sensitivity(self) -> int: ...
	def SetSelectionState(self, theState: SelectMgr_StateOfSelection) -> None: ...
	def SetSensitivity(self, theNewSens: int) -> None: ...
	def UpdateBVHStatus(self, theStatus: SelectMgr_TypeOfBVHUpdate) -> None: ...
	@overload
	def UpdateStatus(self) -> SelectMgr_TypeOfUpdate: ...
	@overload
	def UpdateStatus(self, theStatus: SelectMgr_TypeOfUpdate) -> None: ...

class SelectMgr_SelectionImageFiller(Standard_Transient):
	def Fill(self, theCol: int, theRow: int, thePicked: int) -> None: ...
	def Flush(self) -> None: ...

class SelectMgr_SelectionManager(Standard_Transient):
	def __init__(self, theSelector: SelectMgr_ViewerSelector) -> None: ...
	def Activate(self, theObject: SelectMgr_SelectableObject, theMode: Optional[int] = 0) -> None: ...
	def ClearSelectionStructures(self, theObj: SelectMgr_SelectableObject, theMode: Optional[int] = -1) -> None: ...
	def Contains(self, theObject: SelectMgr_SelectableObject) -> bool: ...
	def Deactivate(self, theObject: SelectMgr_SelectableObject, theMode: Optional[int] = -1) -> None: ...
	def IsActivated(self, theObject: SelectMgr_SelectableObject, theMode: Optional[int] = -1) -> bool: ...
	def Load(self, theObject: SelectMgr_SelectableObject, theMode: Optional[int] = -1) -> None: ...
	def RecomputeSelection(self, theObject: SelectMgr_SelectableObject, theIsForce: Optional[bool] = False, theMode: Optional[int] = -1) -> None: ...
	def Remove(self, theObject: SelectMgr_SelectableObject) -> None: ...
	def RestoreSelectionStructures(self, theObj: SelectMgr_SelectableObject, theMode: Optional[int] = -1) -> None: ...
	def Selector(self) -> SelectMgr_ViewerSelector: ...
	def SetSelectionSensitivity(self, theObject: SelectMgr_SelectableObject, theMode: int, theNewSens: int) -> None: ...
	@overload
	def SetUpdateMode(self, theObject: SelectMgr_SelectableObject, theType: SelectMgr_TypeOfUpdate) -> None: ...
	@overload
	def SetUpdateMode(self, theObject: SelectMgr_SelectableObject, theMode: int, theType: SelectMgr_TypeOfUpdate) -> None: ...
	def Update(self, theObject: SelectMgr_SelectableObject, theIsForce: Optional[bool] = True) -> None: ...
	def UpdateSelection(self, theObj: SelectMgr_SelectableObject) -> None: ...

class SelectMgr_SensitiveEntity(Standard_Transient):
	def __init__(self, theEntity: Select3D_SensitiveEntity) -> None: ...
	def BaseSensitive(self) -> Select3D_SensitiveEntity: ...
	def Clear(self) -> None: ...
	def IsActiveForSelection(self) -> bool: ...
	def ResetSelectionActiveStatus(self) -> None: ...
	def SetActiveForSelection(self) -> None: ...

class SelectMgr_SortCriterion:
	def __init__(self) -> None: ...
	def IsCloserDepth(self, theOther: SelectMgr_SortCriterion) -> False: ...
	def IsHigherPriority(self, theOther: SelectMgr_SortCriterion) -> False: ...

class SelectMgr_ViewClipRange:
	def __init__(self) -> None: ...
	def AddClipSubRange(self, theRange: Bnd_Range) -> None: ...
	def AddClippingPlanes(self, thePlanes: Graphic3d_SequenceOfHClipPlane, thePickRay: gp_Ax1) -> None: ...
	def ChangeUnclipRange(self) -> Bnd_Range: ...
	def GetNearestDepth(self, theRange: Bnd_Range) -> Tuple[bool, float]: ...
	def IsClipped(self, theDepth: float) -> bool: ...
	def SetVoid(self) -> None: ...

class SelectMgr_CompositionFilter(SelectMgr_Filter):
	def ActsOn(self, aStandardMode: TopAbs_ShapeEnum) -> bool: ...
	def Add(self, afilter: SelectMgr_Filter) -> None: ...
	def Clear(self) -> None: ...
	def IsEmpty(self) -> bool: ...
	def IsIn(self, aFilter: SelectMgr_Filter) -> bool: ...
	def Remove(self, aFilter: SelectMgr_Filter) -> None: ...
	def StoredFilters(self) -> SelectMgr_ListOfFilter: ...

class SelectMgr_ViewerSelector3d(SelectMgr_ViewerSelector):
	def __init__(self) -> None: ...
	def ClearSensitive(self, theView: V3d_View) -> None: ...
	@overload
	def DisplaySensitive(self, theView: V3d_View) -> None: ...
	@overload
	def DisplaySensitive(self, theSel: SelectMgr_Selection, theTrsf: gp_Trsf, theView: V3d_View, theToClearOthers: Optional[bool] = True) -> None: ...
	@overload
	def Pick(self, theXPix: int, theYPix: int, theView: V3d_View) -> None: ...
	@overload
	def Pick(self, theXPMin: int, theYPMin: int, theXPMax: int, theYPMax: int, theView: V3d_View) -> None: ...
	@overload
	def Pick(self, thePolyline: TColgp_Array1OfPnt2d, theView: V3d_View) -> None: ...

class SelectMgr_AndFilter(SelectMgr_CompositionFilter):
	def __init__(self) -> None: ...
	def IsOk(self, anobj: SelectMgr_EntityOwner) -> bool: ...

class SelectMgr_AndOrFilter(SelectMgr_CompositionFilter):
	def __init__(self, theFilterType: SelectMgr_FilterType) -> None: ...
	def FilterType(self) -> SelectMgr_FilterType: ...
	def IsOk(self, theObj: SelectMgr_EntityOwner) -> bool: ...
	def SetDisabledObjects(self, theObjects: Graphic3d_NMapOfTransient) -> None: ...
	def SetFilterType(self, theFilterType: SelectMgr_FilterType) -> None: ...

class SelectMgr_OrFilter(SelectMgr_CompositionFilter):
	def __init__(self) -> None: ...
	def IsOk(self, anobj: SelectMgr_EntityOwner) -> bool: ...

#classnotwrapped
class SelectMgr_Frustum: ...

#classnotwrapped
class SelectMgr_FrustumBuilder: ...

#classnotwrapped
class SelectMgr_BaseFrustum: ...

#classnotwrapped
class SelectMgr_TriangularFrustum: ...

#classnotwrapped
class SelectMgr_RectangularFrustum: ...

#classnotwrapped
class SelectMgr_TriangularFrustumSet: ...

#classnotwrapped
class SelectMgr_ToleranceMap: ...

#classnotwrapped
class SelectMgr_ViewerSelector: ...

#classnotwrapped
class SelectMgr_SensitiveEntitySet: ...

# harray1 classes
# harray2 classes
# hsequence classes

selectmgr_ComputeSensitivePrs = selectmgr.ComputeSensitivePrs
